# Investments.pm - The Object Class that provides a Investments Object
#$Id: Main.pm,v 1.31.4.2 2006/08/08 17:06:07 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# # Created by Xperience, Inc.

package Portal::Accounting::Investments::Main;
use strict;
use Portal::Accounting::BaseL;
use Portal::Accounting::Investments::Objects::Search;
use Portal::Accounting::Investments::Objects::LotNode;
use Date::Calc qw(Add_Delta_Days);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.31.4.2 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Investments",
  shareSelect => "Select Lots",
  lotSelect => "Select Lots",
  demo => 'Demo Data',
  regressionLotNode  => 'Regression tests',
  regressionBuy => 'Regression tests',
  regressionSell => 'Regression tests',
  regressionAdj => 'Regression tests',
  regressionReturn => 'Regression tests',
  regressionTransfer => 'Regression tests',
  regressionSelect => 'Regression tests',
  regressionInsert => 'Regression tests',
  regressionMerger => 'Regression tests',
  regressionSpinOff => 'Regression tests',
  regressionStockSplit => 'Regression tests',
  regressionLotSelect => 'Regression tests',
  regressionDelete => 'Regression tests',
);

=head1 NAME

Investments - Object used to build a Investments for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Main;
  my $obj = Portal::Accounting::Investments::Main->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This module gives links to the investment subsystems.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Investments::Main module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Investments State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{objects} = Portal::Accounting::Investments::Objects::Base->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($self->{objects}->error) { $self->error($self->{objects}->errorMessage); }

  # do validation
  if (!$self->Portal::Accounting::Investments::Main::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ($self->numInvalid() > 0 ,, $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }
  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  use Portal::Accounting::DB::Main;
  my %args = ( command => "display", message => "", @_ );
  my $dbm = Portal::Accounting::DB::Main->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
  if ($dbm->error) { $self->error($dbm->errorMessage); return undef; }
  my $command = $args{command};
  my $l = $self->{langObj};
  my $message = $args{message};
  my $doc = $self->prepDoc;
  my $url = $self->{baseURL};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $wat = $doc->encodeString(string => "Investments::WatchList");
  my $cgl = $doc->encodeString(string => "Investments::CGL");
  my $sp = $doc->encodeString(string => "Investments::StockPrices");
  my $port = $doc->encodeString(string => "Investments::Portfolio");
  my $ij = $doc->encodeString(string => "Investments::Journal");
  my $im = $doc->encodeString(string => "Investments::Main");
  my $Buy = $l->map("Buy");
  my $Sell = $l->map("Sell");
  my $Fee = $l->map("Adjustment");
  my $Memo = $l->map("Memo");
  my $Merger = $l->map("Merger");
  my $Return = $l->map("ReturnOfCapital");
  my $Spin = $l->map("SpinOff");
  my $Split = $l->map("StockSplit");
  my $StockTx = $l->map("StockTransfer");
  my $search = Portal::Accounting::Investments::Objects::Search->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $ledgerCode);
  my %form = $search->printForm;
  my @date = $self->getDateArray;
  my $cashSummary = $dbm->getCashAccountSummary( ledgerCode => $ledgerCode, date => "$date[0]-$date[1]-$date[2]", bank => 0);

  my $out = <<"ENDOF";
<div>
  <p>
  <b>#LANG=New Investment Transactions#</b><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=buy">$Buy</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=sell">$Sell</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=adjs">$Fee</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=memo">$Memo</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=acqs">$Merger</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=rtcp">$Return</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=spof">$Spin</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=splt">$Split</a><br>
  <a href="$url?app=$self->{app}&state=$ij&ledgerCode=$ledgerCode&command=new&type=sxfr">$StockTx</a><br>
  </p>
  <p><b>#LANG=Reports#</b>
  <br> <a href="$url?app=$self->{app}&state=$port&command=display&ledgerCode=$ledgerCode">#LANG=Portfolio#</a>
  <br> <a href="$url?app=$self->{app}&state=$cgl&command=display&ledgerCode=$ledgerCode">#LANG=Capital Gains/Losses#</a>
  <br> <a href="$url?app=$self->{app}&state=$wat&command=display&ledgerCode=$ledgerCode">#LANG=Watchlists#</a>
  <br> <a href="$url?app=$self->{app}&state=$sp&command=display&ledgerCode=$ledgerCode">#LANG=Historical# #LANG=Stock Quotes#</a>
  </p>
</div>

  <p><b>Administrative</b><br>
  <a href="$url?app=$self->{app}&state=Accounts&ledgerCode=$ledgerCode&command=new&meta=1&type=3">#LANG=New# #LANG=Account#</a><br>
  <a href="$url?app=$self->{app}&state=SubAccounts&ledgerCode=$ledgerCode&command=list&type=70">#LANG=Edit# #LANG=Stocks#</a><br>
  <a href="$url?app=$self->{app}&state=SubAccounts&ledgerCode=$ledgerCode&command=list&type=57">#LANG=Edit# #LANG=Bonds#</a><br>
  <a href="$url?app=$self->{app}&state=SubAccounts&ledgerCode=$ledgerCode&command=list&type=55">#LANG=Edit# #LANG=Assets#</a><br>
  </p>

ENDOF

  $out = $self->langSubstitute($out);

  $doc->print(<<"ENDOF");
<div style="float: right;">
  $cashSummary
ENDOF
  if ($dbm->error) { $doc->print($dbm->errorMessage); }
  if ($search->error) { $doc->print($search->errorMessage); }
  else { $doc->print(%form); }
  $doc->print(<<"ENDOF");
</div>
 $out
ENDOF
  if ($self->demoCheck)
  {
    $doc->print(<<"DEMO");
<form name="demo">
  <input type="hidden" name="app" value="Accounting" />
  <input type="hidden" name="state" value="Investments::Main" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  <input type="hidden" name="command" value="demo" />
  <input type="submit" value="Input Demo Data" />
</form>
DEMO
  }

  return $doc;
}

sub c_demo
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if ($self->demoCheck)
  {
    $doc->print('<div class="secondary">DEMO DATA<br>');
    use Portal::Accounting::Investments::Objects::Demo;
    my $demo = Portal::Accounting::Investments::Objects::Demo->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
    if ($demo->error) { $doc->print($demo->errorMessage); }
    else
    {
      my %form = $demo->printForm;
      if ($demo->error) { $doc->print($demo->errorMessage); }
      else
      {
        $doc->print(%form);
        $doc->print(<<"ENDOF");
<form name="delete" method="post">
  <p>
  Event<input type="text" name="id" value="" />
  <input type="hidden" name="app" value="Accounting" />
  <input type="hidden" name="state" value="Investments::Journal" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  <input type="hidden" name="command" value="delete" />
  <input type="submit" value="Delete" />
</form>
</div>
ENDOF
      }
    }
  }
  else { $doc->print('<div class="error">Demo is not turned on</div>'); }
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $doc = $self->SUPER::prepDoc(@_);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Investments.js");
  $doc->setJavascriptInclude(file => $self->{jsDir}."tableCollapse.js");
  return $doc;
}

sub mainButton
{
  my $self = shift;
  my $return;
  my $display = $self->{langObj}->map('Investments');
  $return = << "ENDOF";
  <form name="display">
    <input type="submit" value="$display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="Investments::Main">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="command" value="display">
  </form>
ENDOF
  return $return;
}

sub c_regressionLotNode
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $shares = 1;
  my $cost = 100;
  my $account = $self->{input}->{investment1};
  my $account = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $lot = 123456;
  my $r;
  my $yesno;
  my $deletes = 0;
  $doc->print('REGRESSION LOTNODE TEST DATA');
  $doc->print("<div>Using account: $account<br>SubAccount: $stock</div>\n");

 #create LotNodes and a child node and put a child under it.
 #get the childnodes for the top node
 #get the whole tree
 #output should include first node with info and array/list of all leaves with info in addition to whole tree since tree has to be walked anyway.

  use Portal::Accounting::Investments::Objects::LotNode;
  my $l = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($l->error) { $doc->print($l->errorMessage); }

  $doc->print("<div>Testing LotNode->populate . . . ");
  $l->populate(
    "event" => 1,
    "date" => '2007-01-1',
#    "parent" => -1,
#    "journalRef" => -1,
    "subEntity" => 'se',
    "account" => $account,
    "subAccount" => $stock,
    "lot" => $lot,
    "buyLot" => 1,
    "shares" => $shares++,
    "cost" => $cost,
  );
  if ($l->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $l->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing LotNode->insert . . . ");
  $l->insert();
  if ($l->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $l->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  my $m = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($m->error) { $doc->print($m->errorMessage); }
  $doc->print("<div>Testing LotNode->populate . . . ");
  $m->populate(
    "event" => 2,
    "date" => '2007-01-2',
    "parent" => $l->get('id'),
#    "journalRef" => -1,
    "subEntity" => 'se',
    "account" => $account,
    "subAccount" => $stock,
    "lot" => $lot,
    "buyLot" => 1,
    "shares" => $shares++,
    "cost" => $cost,
  );
  if ($m->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $m->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing LotNode->insert . . . ");
  $m->insert();
  if ($m->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $m->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  my $n = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($n->error) { $doc->print($n->errorMessage); }
  $n->populate(
    "event" => 3,
    "date" => '2007-1-3',
    "parent" => $m->get('id'),
#    "journalRef" => -1,
    "subEntity" => 'se',
    "account" => $account,
    "subAccount" => $stock,
    "lot" => $lot,
    "buyLot" => 1,
    "shares" => $shares,
    "cost" => $cost - 10,
  );
  if ($n->error) { $doc->print($n->errorMessage); }
  $n->insert();
  if ($n->error) { $doc->print($n->errorMessage); }

  my $o = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($o->error) { $doc->print($o->errorMessage); }
  $o->populate(
    "event" => 3,
    "date" => '2007-01-3',
    "parent" => $m->get('id'),
#    "journalRef" => -1,
    "subEntity" => 'se',
    "account" => $account,
    "subAccount" => 5001,
    "lot" => 2,
    "buyLot" => 1,
    "shares" => $shares,
    "cost" => 10,
  );
  if ($o->error) { $doc->print($o->errorMessage); }
  $o->insert();
  if ($o->error) { $doc->print($o->errorMessage); }

  my $p = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  my $z = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});

  $doc->print("<div>Testing LotNode->select . . . ");
  $p->set('id', $l->get('id'));
  $p->select();
  if ($p->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $p->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing LotNode->select for parent . . . ");
  $z->set('id', $o->get('id'));
  $z->select();
  if ($z->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $z->errorMessage . '</div>'); }
  else {
    if ($z->get('parent') eq $o->get('parent')) { $doc->print("yes</div>"); }
    else { $doc->print('<font color="red">no: </font>'. $z->get('parent') . ' != '.$o->get('parent').'</div>'); }
  }

  $doc->print("<div>Testing LotNode->tree . . . ");
  my @list = $p->tree;
  if ($p->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $p->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }
  $doc->print("<p>Tree:".join(",",@list)."</p>");

#### clean up lot nodes
  $doc->print("<div>Testing LotNode->delete ".$o->get('id').". . . ");
  $doc->print($o->delete());
  if ($o->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $o->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print($p->walktree($p) );

  $doc->print("<div>Testing LotNode->deleteTree ".$p->get('id').". . . ");
  $doc->print($p->deleteTree() );
  if ($p->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $p->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  return $doc;
}

sub c_regressionSelect
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares = 1;
  my $price;
  my $cost = 100.0;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $lot = 1;
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Select and Delete on each event using manual entry (SQL)');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::Memo;
  use Portal::Accounting::Investments::Objects::StockSplit;
  use Portal::Accounting::Investments::Objects::Merger;
  use Portal::Accounting::Investments::Objects::ReturnOfCapital;
  use Portal::Accounting::Investments::Objects::SpinOff;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql =>
      "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, L, OS, NS, CC, ledger_code, event_ref, comment, sub_e_code) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,?)" ,
    plug => [
      $event->get('type') ,
      $event->get('date') ,
      $event->get('date') ,
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('account'),
      $event->get('stock') ,
      $lot,
      $event->get('price'),
      $event->get('shares'),
      $event->get('commission'),
      $ledgerCode,
      $refs{$type},
      $event->get('comment'),
      $event->get('subEntity'),
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Adjustment";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-2',
    stock => $stock,
    account => $account,
    lot => $lot,
    "comment" => 'adj alala',
    "total" => -5.00,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, L, CC, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $event->get('type') ,
      $event->get('date'),
      $event->get('date'),
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('account'),
      $event->get('stock'),
      $lot,
      $event->get('total'),
      $event->get('ledgerCode'),
      $refs{$type},
      $event->get('comment')
        ]
  );

  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Sell";
  $shares = 100.00;
  $cost = 200.00;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lot => $lot,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql =>
      "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, L, OS, NS, CC, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)" ,
    plug => [
      $event->get('type') ,
      $event->get('date') ,
      $event->get('date') ,
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('account'),
      $event->get('stock') ,
      $lot,
      $event->get('price'),
      $event->get('shares'),
      $event->get('commission'),
      $ledgerCode,
      $refs{$type},
      $event->get('comment'),
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "StockSplit";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-4',
    stock => $stock,
    "comment" => 'split it loo',
    "oldShares" => 1,
    "totalShares" => 2,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql =>
      "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, S, OS, NS, ledger_code, event_ref) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $event->get('type') ,
      $event->get('date') ,
      $event->get('date') ,
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('stock') ,
      $event->get('oldShares') ,
      $event->get('totalShares') ,
      '*',
      $refs{$type},
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Merger";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    oldStock => $stock,
    newStock => $stock2,
    "comment" => 'merger',
    "oldShares" => 1,
    "newShares" => 5,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, OS, NS, ledger_code, event_ref, comment) "
      .  "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $event->get('type') ,
      $event->get('date'),
      $event->get('date'),
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('oldStock'),
      $event->get('newStock'),
      $event->get('oldShares'),
      $event->get('newShares'),
      '*',
      $refs{$type},
      $event->get('comment')
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "ReturnOfCapital";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-6',
    stock => $stock,
    "comment" => 'return of capital',
    "total" => 1.50,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, S, CC, ledger_code, event_ref, comment) "
      .                      "VALUES (?,          ?,         ?,          ?,       ?, ?,  ?,           ?,         ?)",
    plug => [
      $event->get('type') ,
      $event->get('date'),
      $event->get('date'),
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('stock'),
      $event->get('total'),
      '*',
      $refs{$type},
      $event->get('comment')
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "SpinOff";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-7',
    oldStock => $stock2,
    newStock => $stock1,
    "comment" => 'spin it',
    "oldShares" => 1,
    "newShares" => 2,
    cost => 25,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, OS, NS, CC, ledger_code, event_ref, comment) "
    . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $event->get('type') ,
      $event->get('date') ,
      $event->get('date') ,
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('oldStock') ,
      $event->get('newStock') ,  ###!!!! might be reversed
      $event->get('oldShares') ,
      $event->get('newShares') ,
      ($event->get('cost') / 100.0) ,
      '*',
      $refs{$type},
      $event->get('comment')
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "StockTransfer";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $lot,
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 5.0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, L, NS, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $event->get('type') ,
      $event->get('date'),
      $event->get('date'),
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('account'),
      $event->get('stock'),
      $lot,
      (0.0 - $event->get('shares')),
      $event->get('ledgerCode'),
      $refs{$type},
      $event->get('comment')
    ]
  );
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, sub_e_code, A, S, L, NS, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      30,
      $event->get('date'),
      $event->get('date'),
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('subEntity'),
      $event->get('account2'),
      $event->get('stock'),
      $lot + 1 ,
      $event->get('shares'),
      $event->get('ledgerCode'),
      $refs{$type},
      $event->get('comment')
    ]
  );

  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Memo";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-8',
    stock => $stock,
    "comment" => 'memeo meme meo meo meomoe moe',
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  ##first insert data for adjustment manually and call propagate functions manually to make sure they work
  $doc->print("<div>Testing manual entry of $type . . . ");
  $refs{$type} = $event->nextSecurityEventRef;
  $self->{dbHandle}->write(
    sql =>
      "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, S, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?)" ,
    plug => [
      'memo' ,
      $event->get('date') ,
      $event->get('date') ,
      $event->nextSecurityEventCounter($event->get('date') ),
      $event->get('stock') ,
      $ledgerCode,
      $refs{$type},
      $event->get('comment'),
    ]
  );
  if ($self->{dbHandle}->error) { $doc->print("no: ".  $self->{dbHandle}->errorMessage);  $self->dbHandle->rollback; }
  else {$doc->print("yes"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");


  return $doc;
}

sub c_regressionInsert
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares = 1;
  my $price;
  my $cost = 100.0;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $lot = 1;
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for INSERT and DELETE on each event');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::Memo;
  use Portal::Accounting::Investments::Objects::StockSplit;
  use Portal::Accounting::Investments::Objects::Merger;
  use Portal::Accounting::Investments::Objects::ReturnOfCapital;
  use Portal::Accounting::Investments::Objects::SpinOff;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "ReturnOfCapital";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-6',
    stock => $stock,
    "comment" => 'return of capital',
    "total" => 1.50,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "SpinOff";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-7',
    oldStock => $stock2,
    newStock => $stock1,
    "comment" => 'spin it',
    "oldShares" => 1,
    "newShares" => 2,
    cost => 25,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "StockSplit";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-4',
    stock => $stock,
    "comment" => 'split it loo',
    "oldShares" => 1,
    "totalShares" => 2,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>: '.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Memo";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-8',
    stock => $stock,
    "comment" => 'memeo meme meo meo meomoe moe',
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "Merger";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    oldStock => $stock,
    newStock => $stock2,
    "comment" => 'merger',
    "oldShares" => 1,
    "newShares" => 5,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $self->dbHandle->rollback;
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-6-1',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );

  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  my $buy = $event2;
################################################
  $doc->print("<hr>");
  $type = "Adjustment";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-6-2',
    stock => $stock,
    account => $account,
    lot => $objs{Buy}[1]->get('lot'),
    "comment" => 'adj alala',
    "total" => -5.00,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no:</font>'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event->deleteEvents;
  if ($event->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $doc->print("<hr>");
  $type = "Sell";
  $shares = 100.00;
  $cost = 200.00;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-6-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lot => $objs{Buy}[1]->get('lot'),
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $doc->print("<hr>");
  $type = "StockTransfer";
  my $sxfr;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$sxfr = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-6-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $objs{Buy}[1]->get('lot'),
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 5.0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2). . . ");
  $sxfr->populate(
    date => '2007-6-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lots => $objs{Buy}[1]->get('lot').'=10',
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 10.0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event->deleteEvents;
  if ($event->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $r = $sxfr->insert;
  $refs{$type} = $sxfr->get('id');
  if ($sxfr->error) { $doc->print("<font color=\"red\">no</font>: $r".  $sxfr->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $sxfr->resetError;
#   $doc->print("<div>Testing $type"."->delete fail on lots. . . ");
# #  $r = $sxfr->delete;
#   if (!$sxfr->error) { $doc->print('<font color="red">no: '.$r.'</font></div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");
#trashes the db handle and cant make buy work

  #####################dleete buy`
  $doc->print("<hr>");
  $self->dbHandle->rollback;
  $buy->resetError;
  $doc->print("<div>Testing Buy"."->delete . . . ");
  $r = $buy->deleteEvents;
  if ($buy->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $buy->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

  return $doc;
}

sub c_regressionSpinOff
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Sell Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

################################################
  $type = "SpinOff";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    oldStock => $stock,
    newStock => $stock2,
    "comment" => 'spin',
    "oldShares" => 1,
    "newShares" => 5,
    cost => 25,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");

  ###################33
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle});
  my $newnode = $lotNode->wholeTree( lot => $objs{'Buy'}[0]->get('lot'));
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  elsif (not defined $newnode) { $doc->print('newnode is not defined!'.$lotNode->debugString); return $doc; }
  elsif ($newnode->error) { $doc->print($newnode->errorMessage); }
  else
  {
    my $id = $newnode->get('id');
    $doc->print($id);
    $doc->print($newnode->walktree());
  }

  return $doc;
}

sub c_regressionStockSplit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  $doc->print('REGRESSION TEST for StockSplit events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::StockSplit;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

################################################
  $type = "StockSplit";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    stock => $stock,
    "comment" => 'split',
    "oldShares" => 1,
    "totalShares" => 5,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");

  $doc->print("<div>Testing lotNode select for buy and split . . . <br> ");
  my $d = $event->get('date');
  my $lot = $objs{'Buy'}[0]->get('lot');
  my @d = split('-',$d);
  my @e = Add_Delta_Days(@d, -1);
  my $e = join('-',@e);
  my @f = Add_Delta_Days(@d, 1);
  my $f = join('-',@f);
  my $a = $lotNode->balance(lot => $lot, date => $e);
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  if ($a->error) { $doc->print($a->errorMessage); }
  my $a_ = $a->get('shares');
  my $b = $lotNode->balance(lot => $lot, date => $d);
  my $b_ = $b->get('shares');
  my $c = $lotNode->balance(lot => $lot, date => $f);
  my $c_ = $c->get('shares');
  $doc->print(<<"FOO");
  Balances for lot $lot<br>
<table>
  <tr><td>$e</td><td>$a_</td></tr>
  <tr><td>$d</td><td>$b_</td></tr>
  <tr><td>$f</td><td>$c_</td></tr>
</table>
</div>
FOO

   $event = $objs{'Buy'}[0];
#   $event->set('memo', 'second buy after split');
   $event->set('id','');
   $event->set('counter', '');
   $doc->print("<div>Testing non chronological buy->insert for split . . . ");
   $r = $event->insert;
#   $refs{$type."2"} = $event->get('id');
   if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
   else {$doc->print("yes: $r"); $self->dbHandle->commit; }
   $doc->print("</div>");

  $doc->print("<div>Testing lotNode select for non chronological buy and split . . . <br> ");
  my $lot = $event->get('lot');  #get lot from buy event just entered
  my $a = $lotNode->balance(lot => $lot, date => $e);
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  if ($a->error) { $doc->print($a->errorMessage); }
  my $a_ = $a->get('shares');
  my $b = $lotNode->balance(lot => $lot, date => $d);
  my $b_ = $b->get('shares');
  my $c = $lotNode->balance(lot => $lot, date => $f);
  my $c_ = $c->get('shares');
  $doc->print(<<"FOO");
  Balances for lot $lot<br>
<table>
  <tr><td>$e</td><td>$a_</td></tr>
  <tr><td>$d</td><td>$b_</td></tr>
  <tr><td>$f</td><td>$c_</td></tr>
</table>
</div>
FOO
  return $doc;
}

sub c_regressionMerger
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Sell Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

################################################
  $type = "Merger";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    oldStock => $stock,
    newStock => $stock2,
    "comment" => 'merger',
    "oldShares" => 1,
    "newShares" => 5,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");

  ###################33
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle});
  my $newnode = $lotNode->wholeTree( lot => $objs{'Buy'}[0]->get('lot'));
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  elsif (not defined $newnode) { $doc->print('newnode is not defined!'.$lotNode->debugString); return $doc; }
  elsif ($newnode->error) { $doc->print($newnode->errorMessage); }
  else
  {
    my $id = $newnode->get('id');
    $doc->print($id);
    $doc->print($newnode->walktree());
  }

  return $doc;
}

sub c_regressionTransfer
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $date = $self->{input}->{demodate};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Stock Transfer events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

################################################
  $type = "StockTransfer";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $objs{'Buy'}[0]->get('lot'),
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 5.0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2) . . . ");
  $event2->populate(
    date => '2007-1-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $objs{'Buy'}[0]->get('lot'),
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 15.0,
  );
  if ($event2->error) { $doc->print('<font color="red">no: </font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $event2->resetError;
  $doc->print('<div style="background: #cccccc">Failure Testing '.$type."->insert using lots . . . ");
  $event2->set('lots',$objs{Buy}[1]->get('lot').'=25');
  $r = $event2->insert;
  if (!$event2->error) { $doc->print("<font color=\"red\">no</font>: $r");  }
  else {$doc->print("yes: $r<br>".$event2->errorMessage); $self->dbHandle->rollback; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print('<div style="background: #cccccc">Failure Testing '.$type."->insert too many using lots . . . ");
  $event2->set('lots',$objs{Buy}[1]->get('lot').'=5000');
  $r = $event2->insert;
  if (!$event2->error) { $doc->print("<font color=\"red\">no</font>: $r");  }
  else {$doc->print("yes: $r<br>".$event2->errorMessage); $self->dbHandle->rollback; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $event2->set('lots',$objs{Buy}[1]->get('lot').'=15');
  $r = $event2->insert;
  if ($event2->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event2->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");

  ###################33
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle});
  my $newnode = $lotNode->wholeTree( lot => $objs{'Buy'}[0]->get('lot'));
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  elsif (not defined $newnode) { $doc->print('newnode is not defined!'.$lotNode->debugString); return $doc; }
  elsif ($newnode->error) { $doc->print($newnode->errorMessage); }
  else
  {
    my $id = $newnode->get('id');
    $doc->print($id);
    $doc->print($newnode->walktree());
  }

  return $doc;
}

sub c_regressionAdj
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Sell Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");
#
################################################
  $type = "Adjustment";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-2',
    stock => $stock,
    account => $account,
    lot => $objs{'Buy'}[0]->get('lot'),
    "comment" => 'adj alala',
    "total" => -5.00,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no:</font>'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

  return $doc;
}

sub c_regressionBuy
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-1',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  return $doc;
}

sub c_regressionReturn
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Sell Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::ReturnOfCapital;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
  $doc->print("<hr>");

################################################
  $type = "ReturnOfCapital";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-2',
    stock => $stock,
    "comment" => 'return to sender',
    "total" => 2.00,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert under cost . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no:</font>'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->insert over cost . . . ");
  $event->set('date', '2007-5-1');
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no:</font>'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  return $doc;
}

sub c_regressionSell
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for Buy Sell Adjustment events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;
  use Portal::Accounting::Investments::Objects::Adjustment;
  use Portal::Accounting::Investments::Objects::StockTransfer;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");
#
################################################
  $type = "Sell";
  $shares = 50.00;
  $cost = 500.00;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lot => $objs{'Buy'}[0]->get('lot'),
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2) . . . ");
  $event2->populate(
    date => '2007-1-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lots => $objs{'Buy'}[0]->get('lot')."=$shares",
  );
  if ($event2->error) { $doc->print('<font color="red">no: </font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $r = $event2->insert;
  $refs{$type} = $event2->get('id');
  if ($event2->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event2->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");


  return $doc;
}

sub c_regressionLotSelect
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  $doc->print('REGRESSION TEST for LotSelect');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::Sell;

#   my $foo = $self->c_lotSelect;
#   if ($self->error) { $doc->print($self->errorMessage); return $doc; }
#   $doc->print('<br><textarea cols="80" rows="10">'.$foo->{body}.'</textarea><br><hr>');

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");
#
################################################
  $type = "Sell";
  $shares = 50.00;
  $cost = 500.00;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lot => $objs{'Buy'}[0]->get('lot'),
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2) . . . ");
  $event2->populate(
    date => '2007-1-3',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lots => $objs{'Buy'}[0]->get('lot')."=$shares",
  );
  if ($event2->error) { $doc->print('<font color="red">no: </font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $r = $event2->insert;
  $refs{$type} = $event2->get('id');
  if ($event2->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event2->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

  ###################33
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle});
  my $newnode = $lotNode->wholeTree( lot => $objs{'Buy'}[0]->get('lot'));
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  elsif (not defined $newnode) { $doc->print('newnode is not defined!'.$lotNode->debugString); return $doc; }
  elsif ($newnode->error) { $doc->print($newnode->errorMessage); }
  else
  {
    my $id = $newnode->get('id');
    $doc->print($id);
    $doc->print($newnode->walktree());
  }
  return $doc;
}

sub c_regressionDelete
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if (!$self->demoCheck) { $doc->print('<p>Test data can only be entered if you have demo mode turned on! (might also need to be an admin)</p>'); return $doc; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $shares;
  my $price;
  my $cost;
  my $account = $self->{input}->{investment1};
  my $account2 = $self->{input}->{investment2};
  my $stock = $self->{input}->{stock1};
  my $stock1 = $self->{input}->{stock1};
  my $stock2 = $self->{input}->{stock2};
  my $commission;
  my $r;
  my $yesno;
  my $event;
  my $event2;
  my $type;
  my %refs;
  my %objs;
  my $date;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  $doc->print('REGRESSION TEST for StockSplit events');

  use Portal::Accounting::Investments::Objects::Buy;
  use Portal::Accounting::Investments::Objects::StockSplit;

################################################
  $type = "Buy";
  $shares = 100.00;
  $cost = 200.00;
  $commission = 1.0;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-01',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo buy',
    "shares" => $shares,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost + $commission),
    "commPerShare" => 0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

################################################
  $type = "StockSplit";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    stock => $stock,
    "comment" => 'split',
    "oldShares" => 1,
    "totalShares" => 5,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#   $event2->resetError;
#   $doc->print("<div>Testing $type"."->delete . . . ");
#   $r = $event2->delete;
#   if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
#   else { $doc->print("yes: $r</div>"); }
#   $doc->print("<hr>");

  $doc->print("<div>Testing lotNode select for buy and split . . . <br> ");
  my $d = $event->get('date');
  my $lot = $objs{'Buy'}[0]->get('lot');
  my @d = split('-',$d);
  my @e = Add_Delta_Days(@d, -1);
  my $e = join('-',@e);
  my @f = Add_Delta_Days(@d, 1);
  my $f = join('-',@f);
  my $a = $lotNode->balance(lot => $lot, date => $e);
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  if ($a->error) { $doc->print($a->errorMessage); }
  my $a_ = $a->get('shares');
  my $b = $lotNode->balance(lot => $lot, date => $d);
  my $b_ = $b->get('shares');
  my $c = $lotNode->balance(lot => $lot, date => $f);
  my $c_ = $c->get('shares');
  $doc->print(<<"FOO");
  Balances for lot $lot<br>
<table>
  <tr><td>$e</td><td>$a_</td></tr>
  <tr><td>$d</td><td>$b_</td></tr>
  <tr><td>$f</td><td>$c_</td></tr>
</table>
</div>
FOO

   $event = $objs{'Buy'}[0];
#   $event->set('memo', 'second buy after split');
   $event->set('id','');
   $event->set('counter', '');
   $doc->print("<div>Testing non chronological buy->insert for split . . . ");
   $r = $event->insert;
#   $refs{$type."2"} = $event->get('id');
   if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }
   else {$doc->print("yes: $r"); $self->dbHandle->commit; }
   $doc->print("</div>");

  $doc->print("<div>Testing lotNode select for non chronological buy and split . . . <br> ");
  my $lot = $event->get('lot');  #get lot from buy event just entered
  my $a = $lotNode->balance(lot => $lot, date => $e);
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  if ($a->error) { $doc->print($a->errorMessage); }
  my $a_ = $a->get('shares');
  my $b = $lotNode->balance(lot => $lot, date => $d);
  my $b_ = $b->get('shares');
  my $c = $lotNode->balance(lot => $lot, date => $f);
  my $c_ = $c->get('shares');
  $doc->print(<<"FOO");
  Balances for lot $lot<br>
<table>
  <tr><td>$e</td><td>$a_</td></tr>
  <tr><td>$d</td><td>$b_</td></tr>
  <tr><td>$f</td><td>$c_</td></tr>
</table>
</div>
FOO
  
################################################
  $type = "Sell";
  $shares = 1.00;
  $cost = 100.00;
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-13',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lot => $objs{'Buy'}[0]->get('lot'),
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2) . . . ");
  $event2->populate(
    date => '2007-1-13',
    stock => $stock,
    account => $account,
    "subEntity" => "lll",
    "comment" => 'foo sell',
    "shares" => $shares ,
    "price" => $cost / $shares,
    "commission" => $commission,
    "total" => ($cost - $commission),
    "commPerShare" => 0,
    lots => $objs{'Buy'}[0]->get('lot')."=$shares",
  );
  if ($event2->error) { $doc->print('<font color="red">no: </font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $r = $event2->insert;
  $refs{$type} = $event2->get('id');
  if ($event2->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event2->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#transfer
################################################
  $type = "StockTransfer";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-1-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $objs{'Buy'}[0]->get('lot'),
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 5.0,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->populate (2) . . . ");
  $event2->populate(
    date => '2007-1-8',
    stock => $stock,
    account => $account,
    account2 => $account2,
    lot => $objs{'Buy'}[0]->get('lot'),
    "subEntity" => "lll",
    "comment" => 'transfer stuff ',
    "shares" => 15.0,
  );
  if ($event2->error) { $doc->print('<font color="red">no: </font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
  if ($event->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event2->resetError;
  $doc->print("<div>Testing $type"."->insert using lots . . . ");
  $event2->set('lots',$objs{Buy}[1]->get('lot').'=15');
  $r = $event2->insert;
  if ($event2->error) { $doc->print("<font color=\"red\">no</font>: $r".  $event2->errorMessage);  }
  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $event->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;

#spin off  
################################################
  $type = "SpinOff";
  eval('$event = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  eval('$event2 = Portal::Accounting::Investments::Objects::'.$type.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});');
  $doc->print("<div>Testing $type"."->populate . . . ");
  $event->populate(
    date => '2007-01-05',
    oldStock => $stock,
    newStock => $stock2,
    "comment" => 'spin',
    "oldShares" => 1,
    "newShares" => 5,
    cost => 25,
  );
  if ($event->error) { $doc->print('<font color="red">no: </font>'. $event->errorMessage . '</div>'); }
  else { $doc->print("yes</div>"); }

  $doc->print("<div>Testing $type"."->insert . . . ");
  $r = $event->insert;
  $refs{$type} = $event->get('id');
    if ($event->error) { $doc->print('<font color="red">no</font>:'.$r.  $event->errorMessage);  }

  else {$doc->print("yes: $r"); $self->dbHandle->commit; }
  $doc->print("</div>");

  $doc->print("<div>Testing $type"."->select $refs{$type} . . . ");
  $event2->set('id', $refs{$type});
  $r = $event2->select();
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $doc->print("<div>Testing $type"."->isValid $refs{$type} . . . ");
  if (!$event2->isValid) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }

  $event->resetError;
  $event2->resetError;
  $doc->print($event->printHeaders);
  $doc->print($event->printReport);
  $doc->print($event2->printReport);
  $doc->print('<div>Report Test . . . ');
  if ($event->error || $event2->error)
  {
    $doc->print('<font color="red">no</font>');
    if ($event->error) { $doc->print($event->errorMessage); }
    if ($event2->error) { $doc->print($event2->errorMessage); }
  }
  else { $doc->print("yes"); }
  $doc->print("</div>");
  $objs{$type}[0] = $event;
  $objs{$type}[1] = $event2;



  ###################33
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, ledgerCode => $self->{input}->{ledgerCode}, dbHandle => $self->{dbHandle});
  my $newnode = $lotNode->wholeTree( lot => $objs{'Buy'}[0]->get('lot'));
  if ($lotNode->error) { $doc->print($lotNode->errorMessage); }
  elsif (not defined $newnode) { $doc->print('newnode is not defined!'.$lotNode->debugString); return $doc; }
  elsif ($newnode->error) { $doc->print($newnode->errorMessage); }
  else
  {
    my $id = $newnode->get('id');
    $doc->print($id);
    $doc->print($newnode->walktree());
  }
  
  ################3
  my @key = keys %{$newnode->{children}};
  my $key = $key[0];
  my @events = $newnode->{children}->{$key}->findLocalEventsInTree();
  if (not defined @events) {$doc->print('<div>EVENTs array is not defined </div>');}
  if ($newnode->error) { $doc->print($newnode->errorMessage); }
  $doc->print('<div style="background: #eeffaa;">Events from first child '.$newnode->{children}->{$key}->get('id').':<br>'.join('<br>',@events)."</div>");
  
  #################3333
  $type = 'Buy';
  $event2 = $objs{$type}[0];
  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  
  $event2 = $objs{$type}[1];
  $event2->resetError;
  $doc->print("<div>Testing $type"."->delete . . . ");
  $r = $event2->deleteEvents;
  if ($event2->error) { $doc->print('<font color="red">no: '.$r.'</font>'. $event2->errorMessage . '</div>'); }
  else { $doc->print("yes: $r</div>"); }
  
  return $doc;
}


=item JSON lotSelect()

  Returns a javascript object representing the lots for the given data
  The input hash should include:
    account
    stock
    date

=cut
sub c_lotSelect
{
  my $self = shift;
  my $date = $self->{input}->{date};
  my $account = $self->{input}->{account};
  my $subAccount = $self->{input}->{stock};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $doc = HTMLObject::Normal->new;
  $doc->setContentType('text/plain');
  use Portal::Accounting::Investments::Objects::LotNode;
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $ledgerCode);
  if ($lotNode->error)
  {
    $doc->print("ERROR('".$lotNode->errorMessage."');");
    return $doc;
  }
  #get all lots for sub account sent in
  my @lotNodes = $lotNode->list(account => $account, stock => $subAccount, date => $date);
  if ($lotNode->error())
  {
    $doc->print("ERROR('".$lotNode->errorMessage."');");
    return $doc;
  }
  $doc->print("var obj = new Array();");
  my $lotCount = 0;
  if (defined @lotNodes)
  {
    for ($lotCount=0; $lotCount < scalar @lotNodes; $lotCount++)
    {
      $doc->print("obj[$lotCount] = '" . $lotNodes[$lotCount]->get('lot') . "';\n");
    }
  }
  if ($lotCount == 0)
  {
    $doc->print("ERROR('<p><font color=\"red\">There were no lots found for this Account $account and Security $subAccount as of $date.</font></p>');");
  }
  return $doc;
}

sub c_shareSelect
{
  my $self = shift;
  use Date::Calc qw(Add_Delta_Days);
  #subtract one day from date so that user cannot sell on same day as buy.  this way they wont show up in list.
#  my $date = join('-', Add_Delta_Days(split('-', $self->{input}->{demodate}), -1));
  my $date = $self->{input}->{date};
  my $account = $self->{input}->{account};
  my $subAccount = $self->{input}->{stock};
  my $doc = $self->prepDoc;
  my $body = "";
  my $first;
  $doc->setFocus("body");
  my $lotNode = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($lotNode->error) { $doc->print($lotNode->errorMessage);  return $doc;  }
  #get all lots for account, sub account pair sent in
  my @lotNodes = $lotNode->list(account => $account, stock => $subAccount, date => $date);
  if ($lotNode->error())
  {
    $doc->print("<font color=\"red\">SecurityDBObject had error</font><br>\n");
    $doc->print( $lotNode->errorMessage());
    return $doc;
  }
  if (not defined @lotNodes || scalar @lotNodes == 0)
  {#there were no lots or there was a problem
    # lookup the account and sub account names.
    my $accountObj = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $accountObj->set("id", $account);
    $accountObj->select();
    if ($accountObj->error)
    {
      $doc->print($accountObj->errorMessage);
      return $doc;
    }
    my $accountName = $accountObj->get("name");
    use Portal::Accounting::Investments::Objects::Stock;
    my $stockObj = Portal::Accounting::Investments::Objects::Stock->new(langObj => $self->langObj, dbHandle => $self->dbHandle);
    $stockObj->set("id",$subAccount);
    $stockObj->select;
    if ($stockObj->error)
    {
      $doc->print($stockObj->errorMessage);
      return $doc;
    }
    my $subAccountName = $stockObj->get("name");
    $body .= "<p><font color=\"red\">#LANG=There were no lots found#: $accountName $subAccountName $date</font></p>";
  }
  else
  {
    $body .= <<"ENDOF";
<p>#LANG=numOfShares#</p>
<form name="main">
  <table width="100%" class="secondary">
  <thead><tr>
    <th class="account">Account</th>
    <th class="stock">Stock</th>
    <th class="lot">#LANG=Lot#</th>
    <th class="date">#LANG=Date#</th>
    <th class="subEntity">#LANG=Sub Entity#</th>
    <th class="sharesAvailable">#LANG=Shares Avail#</th>
    <th class="cost">#LANG=Cost#</th>
    <th class="costPerShare">#LANG=Cost#/#LANG=Share#</th>
    <th class="shares">#LANG=Shares#</th>
    <th class="chosenCost">#LANG=Chosen Cost#</th>
  </tr></thead>
  <tbody>
ENDOF
    my $costPerShare;
    my $totalShares = 0;
    my $sharesTotalCost = 0;
    foreach my $index (@lotNodes)
    {
      my $style = "full";
      my $lot = $index->get('lot');
      if ($first eq '') { $first = $lot; }
      my $date = '';
      my $sube = '';
      #lookup buy date for each lot
      my $sql = "SELECT trans_date, sub_e_code FROM sec_event_tb WHERE event_code = 'buy' AND l = ?";
      my $sth = $self->dbHandle->read(sql => $sql, plug => [$index->get('buyLot')]);
      if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); next; }
      if (my $data = $sth->fetchrow_hashref)
      {
        $date = $data->{trans_date};
        $sube = $data->{sub_e_code};
      }
      else { $date = $sql; $sube = $index->get('buyLot'); }
      #format numbers to look nicer
      my $shares = sprintf("%.d", $index->get('shares'));
      if( eval($index->get('shares') - $shares) > 0.0) { $shares = $index->get('shares'); }
      my $cost = sprintf("%.02f", $index->get('cost'));
      if ($shares != 0.0) { $style = "empty"; } #hide this row if the share balance is 0

      $costPerShare = sprintf("%.02f", ($index->get('cost') / $index->get('shares')) );
      my $account = $index->get('account');
      my $stock = $index->get('subAccount');
      $body .= <<"ENDOF";
    <tr style="$style">
      <td class="account">$account</td>
      <td class="stock">$stock</td>
      <td class="lot">
        <a href="#" style="color: blue; " onclick="document.main.cs$lot.value='$shares'; updateCost('$lot'); updateTotal(); writeLots(); return false;">$lot</a>
      </td>
      <td class="date">$date</td>
      <td class="subEntity">$sube<input type="hidden" name="subEntity" value="$sube" /></td>
      <td class="sharesAvailable">
        <a href="#" style="color: blue; " onclick="document.main.cs$lot.value='$shares'; updateCost('$lot'); updateTotal(); writeLots(); return false;">$shares</a>
        <input type="hidden" name="s$lot" value="$shares" />
      </td>
      <td class="cost">$cost<input type="hidden" name="c$lot" value="$cost" /></td>
      <td class="costPerShare">$costPerShare</td>
      <td class="shares"><input align="right" type="text" name="cs$lot" size="10" value="0" onChange="checkMax('$lot'); updateCost('$lot'); updateTotal(); writeLots();"></td>
      <td class="chosenCost"><span id="cc$lot">0.00</span></td>
    </tr>
ENDOF
      $doc->setFocus("javascript");
      $doc->print("shareArray[\"$lot\"]=$shares;\n");
      $totalShares += $shares;
      $sharesTotalCost += $cost;
    }
    $sharesTotalCost = sprintf("%.02f", $sharesTotalCost);
    $body .= <<"ENDOF";
    <tr>
      <th class="account">&nbsp;</th>
      <th class="stock">&nbsp;</th>
      <th colspan="3" class="total">Total</th>
      <th class="sharesAvailable">$totalShares</th>
      <th class="cost">$sharesTotalCost</th>
      <th class="costPerShare">&nbsp;</th>
      <th class="shares"><span id="totalShares">0.0</span></th>
      <th class="chosenCost"><span id="totalCost">0.0</span></th>
    </tr>
    </tbody>
  </table>
  <input type="button" value="#LANG=ALL#" onClick="selectAll(); updateTotal(); writeLots(); ">
</form>
ENDOF
  }
#   $body .= "<form>\n";
#   $body .= "<input type=\"button\" value=\"#LANG=Close#\" onClick=\"self.close();\">\n";
#   $body .= "</form>\n";
  $doc->setFocus('body');
  $doc->print($self->langSubstitute($body));
  $doc->print(css => [
    ".account { text-align: left; display: none; }",
    ".stock { text-align: left; display: none; }",
    ".lot { text-align: left; }",
    ".subEntity { text-align: left; }",
    ".date { text-align: left; }",
    ".sharesAvailable { text-align: right; }",
    ".cost { text-align: right; }",
    ".costPerShare { text-align: right; }",
    ".shares { text-align: right; }",
    ".chosenCost { text-align: right; }",
    ".total { text-align: right; }",
  ] );
  if ($first ne '') { $doc->setOnload("document.main.cs$first.focus();"); }
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut