# Adjustment.pm - The Object Class that provides a Adjustment Object
#$Id: Adjustment.pm,v 1.16 2006/05/12 14:35:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Adjustment;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.16 $';

=head1 NAME

Adjustment

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Adjustment;
  my $obj = Portal::Accounting::Investments::Objects::Adjustment->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  Adjustment is the class used to manage Adjustment security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting Adjustment events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::Adjustment module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "adjs";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] , -Value => $today,
      onchange => 'fetchLots(this.form);',
    },
    "account" => { -Label => $l->map("in")." ".$l->map("account"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts}, -Value => "",
      -NoSelectByDefault => 1,
      onchange => 'fetchLots(this.form);',
      },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}, -Value => "",
      -NoSelectByDefault => 1,
      onchange => 'fetchLots(this.form);',
      },
    "lot" => { -Type => "select", -DataType => "int", -Label => $l->map("Lot"), -Options => { names => [], values => []} },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "total" => { -Type => "text", -DataType => "float", -Label => $l->map("adjustCost"), size => 8 },
  };
  $self->{data}->{report} = {
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "lot" => { -Type => "text", -ReadOnly => 1, },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -ReadOnly => 1, },
    "date" => {-Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today, -ReadOnly => 1, },
    "account" => {-Type => "select", -DataType => "int",
      -Options => $self->{accounts} , -ReadOnly => 1,  },
    "stock" => { -Type => "select", -DataType => "int",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "subEntity" => { -Type => "select", -DataType => "int",
      -Options => $self->{subEntities}, , -ReadOnly => 1,},
    "comment" => { -Type => "text", -DataType => "string", -ReadOnly => 1, },
    "total" => { -Type => "text", -DataType => "float", -ReadOnly => 1,},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $SelectLots = $l->map("SelectLots");
  my $Submit = $l->map("Submit");
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FI=app#
  #FI=state#
  #FI=command#
  #FI=counter#
  #FI=ledgerCode#
  <h4>#LABEL=type##FI=type#</h4>
  <table class="primary">
    <tr>
      <td>#LABEL=date# #FI=date#<br>
        #LABEL=stock# #FI=stock#
        #LABEL=account# #FI=account#
        #LABEL=lot# #FI=lot#
        <input type="button" name="lotsButton" value="*" onclick="fetchLots(this.form);" title="$SelectLots"><br>
        #LABEL=total# \$#FI=total#.<br>
        #LABEL=comment# #FI=comment#<b><font color="green">%</font></b>
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="$Submit" onclick="this.disabled = true;">
  <br><b><font color="green">%</font></b> Optional
</form>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=comment#</td>
    <td class="cost">#FIELD=total#</td>
    <td class="subEntity">#FIELD=subEntity#</td>
    <td class="lot">#FIELD=lot#</td>
$self->{deleteString}   
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date total stock account lot) ],
    optional => [ qw( comment ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         lot => qr/^\d+(\.?)\d*$/,
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date total stock account lot) ],
    optional => [ qw( comment ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         lot => qr/^\d+(\.?)\d*$/,
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };

  $self->{INSERTorder}[0] = [ qw( event_code trans_date counter sub_e_code a s l cc ledger_code event_ref comment ) ] ; #insert order for db column names
  
  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    sub_e_code => 'subEntity',
    a => 'account',
    s => 'stock',
    l => 'lot',
    cc => 'total',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE event_ref = #FIELD=id#";
#  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
#  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE event_ref = #FIELD=id#";
  $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE event_ref = #FIELD=id#";
  
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.
  
=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my %args = (@_ );
  $self->set('id', $self->nextSecurityEventRef());
  $self->set('counter', $self->nextSecurityEventCounter($self->get('date')));

  # retrieve the lot balance entry from the lot_node_tb table for the lot
  #create LotNode
  #set some information in order to retrieve it
  #get Balance
  my $lot = $self->get('lot');
  my $date = $self->get('date');
  #current counter hasn't been entered into lotNode tb yet so it should not be grabbing same node record.  should be getting previous records.
  my $node = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  my $lotbalobj = $node->balance(date => $date, counter => $self->get('counter'), lot => $lot );
#  $self->error("<p>".$lotbalobj->get('id')." Balance ".$lotbalobj->get('date')."</p>");
  if ($node->error)
  {
    $self->error("<p>Failed to retrieve lot balance!</p>" . $node->errorMessage);
  }
  elsif (!defined $lotbalobj)
  {
    $self->error("<p>Lot Balance object is not defined.  Failed to retrieve lot balance!  $lot, $date, ".$self->get('counter')."</p>");
  }
  elsif ($lotbalobj->error)
  {
    $self->error("<p>Failed to retrieve lot balance!</p>" . $lotbalobj->errorMessage);
  }
  elsif ($lotbalobj->get('id') == -1)
  {
    $self->error('<div class="error">No balance found for lot "'.$lot.'" on '.$date.'</div>');
  }
  else
  {
    my $sql = "INSERT INTO sec_event_tb (event_code, trans_date, counter, A, S, L, CC, ledger_code, event_ref, comment) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
  # insert the security event into the database
  $self->{dbHandle}->write(
    sql => $sql,
    plug => [
      $self->get('type'),
      $self->get('date'),
      $self->get('counter'),
      $self->get('account'),
      $self->get('stock'),
      $self->get('lot'),
      $self->get('total'),
      $self->get('ledgerCode'),
      $self->get('id'),
      $self->get('comment')
        ]
      );
    if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
    elsif (!$self->insertEvent($lotbalobj)) { $self->error('<div class="error">Parent insert failed!</div>'); }
  }

  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return $self->get('id');
}

=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $lotNodeObj = shift;
  
  my $dorc = "true";
  my $cord = "false";
  my $date = $self->get('date');

  my $amount = $self->get('total');
  if ($amount < 0)
  {
    $dorc = "false";
    $cord = "true";
  }
  my $costBalance = $lotNodeObj->get('cost') + $amount;
  my $abs = sprintf("%.2f", abs($amount));

  ####check to make sure adj amount does not drop lot below 0 for costBalance
  if ($costBalance < 0) { $self->error("<p>Cost drops below zero when propagating event '".$lotNodeObj->get('event')."' on node '".$lotNodeObj->get('id')."'</p>"); return undef; }
    
  # build a JournalEntry for the data
  my $l = $self->get('ledgerCode');
  my $transobj = Portal::Accounting::Objects::JournalEntry->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, ledgerCode => $l,);
  if ($transobj->error) { $self->error($transobj->errorMessage); return undef; }
  $transobj->set('date', $date);
  $transobj->set('journal', "IS"); # investments journal code hardcoded.
  $transobj->addRow(
    subEntity => $lotNodeObj->get('subEntity'),
    account => $lotNodeObj->get('account'),
    subAccount => $lotNodeObj->get('subAccount'),
    dorc => "$dorc",
    amount => $abs,
    comment => $self->get('comment'),
    counter => 0,
  );
  if ($transobj->error) { $self->error("<br>FOO".$transobj->errorMessage); }
  $transobj->addRow(
    subEntity => $lotNodeObj->get('subEntity'),
    account => $lotNodeObj->get('account'),
    dorc => "$cord",
    amount => $abs,
    comment => $self->get('comment'),
    counter => 1,
  );
  # insert the transaction into the database
  $transobj->isValid;
  if ($transobj->error)
  {
    $self->error("<p>Invalid JournalEntry create for adjs!</p>" . $transobj->errorMessage);
    return undef;
  }
  my $refNum = $transobj->insert();
  if ($transobj->error)
  {
    $self->error("<p>Error inserting journal transaction for adjs!</p>" . $transobj->errorMessage);
    return undef;
  }
  
  # insert a new lot node into the database
  my $newLotNodeObj = Portal::Accounting::Investments::Objects::LotNode->new(ledgerCode => $self->get('ledgerCode'), langObj => $self->{langObj}, dbHandle => $self->dbHandle);
  $newLotNodeObj->populate(
    date => $date,
    parent => $lotNodeObj->get('id'),
    event => $self->get('id'),
    journalRef => $refNum,
    subEntity => $lotNodeObj->get('subEntity'),
    account => $lotNodeObj->get('account'),
    subAccount => $lotNodeObj->get('subAccount'),
    lot => $lotNodeObj->get('lot'),
    buyLot => $lotNodeObj->get('buyLot'),
    cost => sprintf("%.2f", $costBalance),
    shares => sprintf("%.6f", $lotNodeObj->get('shares'))
  );
  $newLotNodeObj->insert;
  if ($newLotNodeObj->error)
  {
    $self->error($newLotNodeObj->errorMessage);
    return undef;
  }

  return $self->SUPER::propagate($newLotNodeObj);
}

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.
  
=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__


=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut