# Demo.pm - The Object Class that provides a Demo Object
#$Id: Demo.pm,v 1.2.6.1 2006/08/08 17:06:08 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Demo;
use strict;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Investments::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Objects::JournalEntry;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.2.6.1 $';

=head1 NAME

Demo - Object used to build a Demo Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Demo;
  my $DemoObj = Portal::Accounting::Investments::Objects::Demo->new(   );
  if ($DemoObj->error())
  {
    die $DemoObj->errorMessage();
  }

=head1 DESCRIPTION

Demo is a Demo class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Investments::Objects::Demo.
    See Portal::Accounting::Investments::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );
  $self->{new} = 1;
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  $self->{object} = "Portal::Accounting::Objects::JournalEntry";
  my @date = Today();
  my $l = $self->{langObj};
  if (! defined $l) { $self->error('langObj is screweed!'); return $self; }
  my $between = $l->map('between');
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $st = Portal::Accounting::Investments::Objects::Stock->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  if ($acc->error) { $self->error($acc->errorMessage); }
  if ($st->error) { $self->error($st->errorMessage); }

  my $all = $l->map('ALL');
  $acc->set('type', '3');
  my $i = $acc->selectOptionsByType();
  my $s = $st->selectOptions;
  if ($acc->error) { $self->error($acc->errorMessage); }
  if ($st->error) { $self->error($st->errorMessage); }
  foreach my $me ($i, $s)
  {
    if (not defined $me) { $me = { names => [], values => []}; }
  }
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Investments::Main" },
    "command" => { -Type => "select",  -Options => {
        names => [ 'Buy', 'Sell', 'Delete', 'Adjustment', 'Return of Capital', 'Transfer', 'Select', 'Insert', 'Merger', 'Spin Off', 'Stock Split', 'Lot Selection', 'Lot Node Inserts', ],
        values => [ qw( regressionBuy regressionSell regressionDelete regressionAdj regressionReturn regressionTransfer regressionSelect regressionInsert regressionMerger regressionSpinOff regressionStockSplit regressionLotSelect regressionLotNode ) ],
      },
    },
    "confirmDemo" => { -Type => "hidden", -Value => "1" , -DataType => 'int'},
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    'demodate' => { -Type => 'date-picker', -DataType => 'string', -year => '2006', -Label => 'Date', },
    "investment1" => { -Label => "Investment", -Type => "select", -DataType => "int",
      -Options => $i,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "investment2" => { -Label => "Investment", -Type => "select", -DataType => "int",
      -Options => $i,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "stock1" => { -Label => "Stock", -Type => "select", -DataType => "int",
      -Options => $s,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "stock2" => { -Label => "Stock", -Type => "select", -DataType => "int",
      -Options => $s,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "stock3" => { -Label => "Stock", -Type => "select", -DataType => "int",
      -Options => $s,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    lot => { -Type => 'text', -DataType => 'int', -Label => "Lot", },
  };
   $self->{template}->{main} = <<"ENDOF";
<div class="title">Demo data and Regression Tests</div>

<p>Choose data from this list and enter data here.  Then click the submit button if you are sure that you want to enter demo data into these books.</p>

<form #FORMARGS# class="primary" target="_new">
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  TEST: #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=confirmDemo#
  
<div style="clear: both; float: left; width: 10em; ">#LABEL=demodate#</div><div> #FIR=demodate#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=investment1#</div><div> #FIR=investment1#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=investment2#</div><div> #FIR=investment2#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=stock1#</div><div> #FIR=stock1#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=stock2#</div><div> #FIR=stock2#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=stock3#</div><div> #FIR=stock3#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=lot#</div><div> #FIR=lot#</div>

  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( investment1 investment2 stock1 stock2 stock3 date) ],
    optional => [ ],
    constraints => {
      investment1 => qr/^\d+$/,
      investment2 => qr/^\d+$/,
      stock1 => qr/^\d+$/,
      stock2 => qr/^\d+$/,
      stock3 => qr/^\d+$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub printReport
{
  my $self = shift;
  my $doc = HTMLObject::Normal->new;
   
  my $investment1 = $self->get('investment1');
  my $investment2 = $self->get('investment2');
  my $stock1 = $self->get('stock1');
  my $stock2 = $self->get('stock2');
  my $stock3 = $self->get('stock3');
#setup objects
  $doc->print(<<"ENDOF");
<p>using accounts:<br>
investment = $investment1<br>
investment = $investment2<br>
And STOCK:
1 = $stock1<br>
1 = $stock2<br>
1 = $stock3<br>
</p><hr>
ENDOF

return $doc;
}

sub populate
{
  my $self = shift;
  $self->{new} = 0;
  return $self->SUPER::populate(@_);
}

sub isNew
{
  my $self = shift;
  return $self->{new};
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
