# Memo.pm - The Object Class that provides a Memo Object
#$Id: Memo.pm,v 1.14 2006/05/12 14:35:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Memo;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.14 $';

=head1 NAME

Memo

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Memo;
  my $obj = Portal::Accounting::Investments::Objects::Memo->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  Memo is the class used to manage Memo security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting Memo events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::Memo module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "memo";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] , -Value => $today },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
  };
  $self->{data}->{report} = {
    "id" => { -Type => "hidden", },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1,},
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] , -Value => $today, -ReadOnly => 1 },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}, -ReadOnly => 1      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1 },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $Submit = $l->map("Submit");
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  <h4>#LABEL=type##FIELD=type#</h4>
  <table class="primary">
    <tr>
      <td>
        #LABEL=date# #FI=date#
        #LABEL=stock# #FI=stock#<br>
        #LABEL=comment# #FI=comment#
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="$Submit" onclick="this.disabled = true;">
</form>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=comment#</td>
$self->{deleteString}
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date stock comment ) ],
    optional => [  ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         stock => qr/^\d+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date stock comment ) ],
    optional => [  ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         stock => qr/^\d+$/,
       },
  };
  $self->{INSERTorder}[0] = [ qw( event_code  trans_date  counter  s ledger_code  event_ref  comment ) ] ;  #insert order for db column names

  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    s => 'stock',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

#   $self->{INSERT}[0] = "INSERT INTO sec_event_tb (event_code, trans_date, counter, comment, S, ledger_code, event_ref)";
#   $self->{INSERT}[0] .= " VALUES ( 'memo',";
#   $self->{INSERT}[0] .= " '#FIELD=date#', '#FIELD=date#' ,";
#   $self->{INSERT}[0] .= " next_sec_event_tb_counter('#FIELD=date#'), ";
#   $self->{INSERT}[0] .= " '#FIELD=comment#' , #FIELD=stock#, '#FIELD=ledgerCode#',";
#   $self->{INSERT}[0] .= " (SELECT nextval('sec_event_tb_event_ref_seq'))";
#   $self->{INSERT}[0] .= " )";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  This routine selects a record from the database and performs any transforms on the data to fill out the object with fields that are not stored in the database but are needed for display and calculations.

=cut

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.

=cut

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.  For memo there should not be any affected nodes.  It is a global event.

=cut
sub delete
{
  my $self = shift;
  my %args = (id => $self->get('id'), @_);
  my $sql = "DELETE FROM sec_event_tb WHERE event_ref = ?";
  $self->{dbHandle}->write(sql => $sql, plug => [$args{id} ]);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); $self->dbHandle->rollback; return 0; }
  $self->dbHandle->commit;
  return 1;
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut