# Merger.pm - The Object Class that provides a Merger Object
#$Id: Merger.pm,v 1.16.4.3 2006/08/08 17:12:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Merger;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.16.4.3 $';

=head1 NAME

Merger

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Merger;
  my $obj = Portal::Accounting::Investments::Objects::Merger->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  Merger is the class used to manage Merger security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting Merger events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::Merger module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "acqs";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] , -Value => $today },
    "oldStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}      },
    "newStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Shares"), size => 8},
    "newShares" => { -Type => "text", -DataType => "float", -Label => $l->map("New")." ".$l->map("Shares"), size => 8},
    "counter" => { -Type => "hidden", -Value => "", -DataType => "integer" },
  };
  $self->{data}->{report} = {
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1, },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] , -Value => $today, -ReadOnly => 1, },
    "oldStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "newStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1, },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Shares"), , -ReadOnly => 1,},
    "newShares" => { -Type => "text", -DataType => "float", -Label => $l->map("New")." ".$l->map("Shares"), , -ReadOnly => 1,},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FI=app#
  #FI=state#
  #FI=command#
  #FI=counter#
  #FI=ledgerCode#
  <h4>#LABEL=type##FI=type#</h4>
  <table class="primary">
    <tr>
      <td>
        #LABEL=date# #FI=date#<br>
        #LANG=youWillHaveTotal#
        #FI=newShares# #LABEL=newShares# #LABEL=newStock# #FI=newStock#<br>
        #LANG=for every#
        #FI=oldShares# #LABEL=oldShares# #LABEL=oldStock# #FI=oldStock#<br>
        100\% #LANG=ofCostwill#<br>
        #LABEL=comment# #FI=comment# <b><font color="green">%</font></b>
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" onclick="this.disabled = true;">
</form>
<p><b><font color="green">%</font></b> Optional</p>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=newStock# #LANG=acquires# #FIELD=oldStock# @ #FIELD=newShares# #LANG=for# #FIELD=oldShares# : #FIELD=comment#</td>
$self->{deleteString}   
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date oldShares oldStock newShares newStock ) ],
    optional => [ qw( comment counter ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         oldShares => 'RE_num_real',
         newShares => 'RE_num_real',
         oldStock => qr/^\d+$/,
         newStock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date oldShares oldStock newShares newStock ) ],
    optional => [ qw( comment counter ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         oldShares => 'RE_num_real',
         newShares => 'RE_num_real',
         oldStock => qr/^\d+$/,
         newStock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  
  $self->{INSERTorder}[0] = [ qw( event_code trans_date counter a s os ns ledger_code event_ref comment ) ] ; #insert order for db column names
  
  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    a => 'newStock',
    s => 'oldStock',
    os => 'oldShares',
    ns => 'newShares',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->get('oldStock') eq $self->get('newStock'))
  {
    $self->invalid('newStock', $self->langObj->map('oldstock') , $self->langObj->map('Companies must be different'));
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.
  
=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my $eventRef = $self->nextSecurityEventRef();
  my $counter = $self->nextSecurityEventCounter($self->get('date'));
  $self->set('id', $eventRef);
  $self->set('counter', $counter);
 #there are race conditions with this approach but not easily fixable since each day starts a new counter at 0
  #perhaps during rewrite a timestamp is used where the new event is put halfway between the other two.
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, trans_date, counter, A, S, OS, NS, ledger_code, event_ref, comment) "
      .                      "VALUES (?,          ?,          ?,       ?, ?, ?,  ?,  ?,           ?,         ?)",
    plug => [
      $self->get('type'),
      $self->get('date'),
      $counter,
      $self->get('newStock'),
      $self->get('oldStock'),
      $self->get('newShares'),
      $self->get('oldShares'),
      '*',
      $eventRef,
      $self->get('comment')
    ]
  );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
  else
  {
    # retrieve the affected lots for all ledgers
    $self->insertIntoLedgers(date => $self->get('date'), counter => $counter, stock => $self->get('oldStock'));
  }

  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return $eventRef;
}

      # for each lot entry, create a new journal entry with the new cost placing the number of shares and the lot number into the memo field
      # create a node for the new lot
      # create a new node for the old lot
      # call propagate on the old lot
      # call propagate on the new lot
=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $old = shift;
  my $new = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $old->get('ledgerCode'));
  my $lot2 = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $old->get('ledgerCode'));
  
  #calculate new lot node data from old lot and data from this eventRef
  # calculate new lot's number of shares
  my $newShares = $self->get('newShares') / $self->get('oldShares') * $old->get('shares');
  # get a new lot number
  my $newLotNumber = $self->nextLotNumber();
  if (!$self->error)
  {
    # create a transaction to generate a new lot moving the cost of the old lot to the new lot
    my $transobj = Portal::Accounting::Objects::JournalEntry->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, ledgerCode => $old->get('ledgerCode') );
    $transobj->set('journal' , $self->{Journal});
    $transobj->set('date' , $self->get('date') );
    $transobj->addRow(
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('oldStock'),
      dorc => "false",
      amount => $old->get('cost'),
      comment => $self->get('newShares')." shares for lot $newLotNumber. ". $self->get('comment'),
    );
    $transobj->addRow(
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('newStock'),
      dorc => "true",
      amount => $old->get('cost'),
      comment => $self->get('newShares')." shares for lot $newLotNumber. ". $self->get('comment'),
    );
    # insert transaction
    if ($transobj->error)
    {
      $self->error($transobj->errorMessage);
      return undef;
    }
    my $refNum = $transobj->insert(system => 1);
    if ($transobj->error)
    {
      $self->error($transobj->errorMessage);
      return undef;
    }

    # create two new lot nodes, one for the new lot that is generated from the event and one for the old lot to decrease
    # its balances (shares, cost)
    $lot2->populate(
      date => $self->get('date'),
      parent => $old->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('newStock'),
      lot => $newLotNumber,
      buyLot => $old->get('buyLot'),
      cost => $old->get('cost'),
      shares => sprintf("%.6f", $newShares),
    );
    if ($lot2->error) { $self->error($lot2->errorMessage); return undef; }
    $new->populate(
      date => $self->get('date'),
      parent => $old->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('oldStock'),
      lot => $old->get('lot'),
      buyLot => $old->get('buyLot'),
      cost => "0.00",
      shares => 0,
    );
    if ($new->error) { $self->error($new->errorMessage); return undef; }
    $new->insert;
    if ($new->error) { $self->error($new->errorMessage); return undef; }
    $lot2->insert;
    if ($lot2->error) { $self->error($lot2->errorMessage); return undef; }
    
    $self->SUPER::propagate($new);
    if (!$self->error) { $self->SUPER::propagate($lot2); }
  }
  return 1;
}

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.
  
=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__


=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut