# StockTransfer.pm - The Object Class that provides a StockTransfer Object
#$Id: StockTransfer.pm,v 1.20.4.1 2006/07/12 20:54:24 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::StockTransfer;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.20.4.1 $';

=head1 NAME

StockTransfer

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::StockTransfer;
  my $obj = Portal::Accounting::Investments::Objects::StockTransfer->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  StockTransfer is the class used to manage StockTransfer security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting StockTransfer events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::StockTransfer module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", input => undef, @_ );

  # instantiate anything unique to this module
  my ($date, $account, $account2, $stock);
  if (defined $args{input})
  {
    $date = $args{input}->{date};
    $account = $args{input}->{account};
    $account2 = $args{input}->{account2};
    $stock = $args{input}->{stock};
  }
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "sxfr";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;
  if ($date eq '') { $date = $today; }

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app},
#      -onload => "fetchShares(document.getElementById('main'));",
    },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -Value => $date, -year => $date[0],
      onchange => "fetchShares(this.form);"
    },
    "account" => { -Type => "select", -DataType => "integer",
      -Options => $self->{accounts},
      -NoSelectByDefault => 1,
      onchange => "fetchShares(this.form);",
      -Value => $account,
    },
    "account2" => { -Type => "select", -DataType => "integer",
      -Options => $self->{accounts},
      -Value => $account2,
    },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks},
      -NoSelectByDefault => 1,
      onchange => "fetchShares(this.form);",
      -Value => $stock,
    },
    "lots" => { -Type => "hidden", -DataType => "text" },
    "lot" => { -Type => "hidden", -DataType => "integer", },
    "lot2" => { -Type => "hidden", -DataType => "integer", },
    "subEntity" => { -Label => $l->map("Sub Entity"), -Type => "select", -DataType => "integer",
      -Options => $self->{subEntities}, -onload => "this.selectedIndex = -1;"      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "shares" => { -Type => "text", -DataType => "float",
      onfocus => "this.form.account2.focus();",
      tabindex => '-1',
      -Label => $l->map("Shares"), size => 8,
    },
  };
  $self->{data}->{report} = {
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "id" => { -Type => "hidden", },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly =>1, },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string", -Value => $today,
      -year => $date[0], -ReadOnly => 1 ,
    },
    "account" => { -Type => "select", -DataType => "integer",
      -Options => $self->{accounts}, -ReadOnly => 1,
    },
    "account2" => { -Type => "select", -DataType => "integer",
      -Options => $self->{accounts}, -ReadOnly => 1      },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}, -ReadOnly => 1,
    },
    "lots" => { -Type => "hidden", -DataType => "text" },
    "lot" => { -Type => "text", -DataType => "text", -ReadOnly => 1},
    "lot2" => { -Type => "text", -DataType => "text", -ReadOnly => 1},
    "subEntity" => { -Label => $l->map("Sub Entity"), -Type => "select", -DataType => "integer",
      -Options => $self->{subEntities},  -ReadOnly => 1      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1 },
    "shares" => { -Type => "text", -DataType => "float",
      -Label => $l->map("Shares"), -ReadOnly => 1,
    },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form id="main" #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=counter#
  <h4>#LABEL=type##FIELD=type#</h4>
  <table class="primary">
    <tr>
      <td>
        #LABEL=date# #FI=date#<br>
        <br>
        #LANG=From# #LANG=Account# #FIELD=account#, #LANG=Stock# #FI=stock#<br>
        #LANG=Transfer#
        #FI=lots#
        <input type="button" name="lotsButton" value="*" onclick="fetchShares(this.form);" title="#LANG=Select Lots#">
        #FI=shares# #LABEL=shares#<br>
        #LANG=To# #LANG=Account# #FI=account2#
        #LABEL=subEntity# #FI=subEntity# <b><font color="green">%</font></b><br>
        #LABEL=comment# #FI=comment# <font color="green">%</font></b>
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" onclick="this.disabled = true;">
  <input type="hidden" name="focusField" value="account2" />
</form>
<p><font color="green">%</font></b> - Optional</p>
<div id="frame">
  <iframe name="sharesselection" align="top" frameborder="0" width="100%" scrolling="yes" Height="600" >
  </iframe>
</div>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail"><i>#FIELD=shares#</i> #LANG=Shares# #LANG=From# <i>#FIELD=account#</i> #LANG=To# <u>#FIELD=account2# (L: #VALUE=lot2#)</u> <span class="comment">#FIELD=comment#</span></td>
    <td class="subEntity">#VALUE=subEntity#</td>
    <td class="lot">#FIELD=lot#</td>
$self->{deleteString}
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date shares stock account account2 ) ],
    optional => [ qw( lots lot subEntity ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         shares => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         account2 => qr/^\d+$/,
         lots => qr/^(\d+=\d+(\.)?(\d+)?)(\;\d+=\d+(\.)?(\d+)?)*$/,
         subEntity => qr/^[\d\w]+$/,
         counter => qr/^\d+$/,
         lot => qr/^\d+$/,
      },
  };
  $self->{profile}->{report} = {
    required => [ qw( date shares stock account account2 ) ],
    optional => [ qw( lots lot subEntity ) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         shares => 'RE_num_real',
         stock => qr/^\d+$/,
         account => qr/^\d+$/,
         account2 => qr/^\d+$/,
         lots => qr/^(\d+=\d+(\.)?(\d+)?)(\;\d+=\d+(\.)?(\d+)?)*$/,
         subEntity => qr/^[\d\w]+$/,
         counter => qr/^\d+$/,
         lot => qr/^\d+$/,
      },
  };

  $self->{INSERTorder}[0] = [ qw( event_code trans_date counter a s l ns  ledger_code event_ref comment sub_e_code ) ] ; #insert order for db column names

  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    a => 'account',
    s => 'stock',
    l => 'lot',
    ns => 'shares',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
    sub_e_code => 'subEntity',
  };  #map database columns to ->data hash, not specified assumes same name

  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE event_ref = #FIELD=id# ORDER BY ns";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  # validate our parameters.
  if ($self->get('lots') eq '')
  {
    if ($self->get('lot') eq '') { $self->missing('lot','Must specify lot or lots.'); }
  }
  if ($self->get('shares') <= 0) { $self->invalid('shares', 'Less than zero'); }
  if ($self->get('account') eq $self->get('account2'))
  {
    $self->invalid(
      $self->langObj->map('Account')." 1",
      $self->langObj->map('Account')." 2",
      $self->langObj->map('Accounts cannot be the same')
    );
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  This routine selects a record from the database and performs any transforms on the data to fill out the object with fields that are not stored in the database but are needed for display and calculations.

=cut
sub select
{
  my $self = shift;
  #need to select two records instead of one
  my ($sqls, $plugs) = $self->substitute(sql => "SELECT");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    if ($self->debug) {  $self->debug($sql);  }
    my $result = $self->{dbHandle}->read($sql);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      return undef;
    }
    if (my $DB = $result->fetchrow_hashref)
    {
      foreach my $key (keys %{$DB})
      {
        my $mapKey = $self->db2data(var => $key, index => $index);
        $self->set($mapKey, $DB->{$key});
      }
    }
    else { $self->error('<div class="error">NO data found for event '.$self->get('id').'</div>'); return 0; }
    if (my $DB = $result->fetchrow_hashref)
    {
      $self->set('account2', $DB->{a});
      $self->set('subEntity', $DB->{sub_e_code});
      $self->set('lot2', $DB->{l});
      $self->set('shares', $DB->{ns});
    }
    else { $self->error('<div class="error">Second record missing for event '.$self->get('id').'!</div>'); return 0; }
  }
  return 0;
}

# what if insert would literally create different event objects for each lot?
# they could be cached inside the first object
# could also be returned as an array

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.

=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my $totalShares = 0;
  my @lots = split(';', $self->get('lots'));
  my $lot;
  my $shares;
  my @events;
  if (scalar @lots == 0)
  {
    #we are not in an object used for forms.  just work with the data in $self->get('lot')
    $lot = $self->get('lot');
    $shares = $self->get('shares');
    my $counter = $self->nextSecurityEventCounter($self->get('date'));
    my $eventRef = $self->nextSecurityEventRef();
    my $newLot = $self->nextLotNumber();
    $self->set('lot2',$newLot);
    $self->set('counter',$counter);
    $self->set('id',$eventRef);

    # retrieve the lot balance entry from the lot_node_tb table for the "old" lot
    my $lotbalobj = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode') );
    $lotbalobj = $lotbalobj->balance(date => $self->get('date'), counter => $counter, lot => $lot );
    if (! defined $lotbalobj)
    {
      $self->error('<div class="error">Failed to retrieve lot balance! obj undefined for '.$lot."! </div>");
    }
    elsif ($lotbalobj->error())
    {
      $self->error($lotbalobj->errorMessage);
    }
    elsif ($lotbalobj->get('id') == -1)
    {
      $self->error('<div class="error">No balance found for lot '.$lot." as of '".$self->get('date')."'!</div>");
    }
    elsif ($lotbalobj->get('shares') < $shares)
    {
      $self->error('<div class="error">Lot '."$lot has only ".$lotbalobj->get('shares')." which is less than the shares to transfer: $shares</div>".$lotbalobj->walktree());
    }
    else
    {
      $self->set('stock', $lotbalobj->get('subAccount'));
      $self->set('account', $lotbalobj->get('account'));
      $self->insertEvent($lotbalobj);
      if (!$self->error)
      {
        # insert two records for a stock transfer
        $self->{dbHandle}->write(
          sql => "INSERT INTO sec_event_tb (event_code, trans_date, counter, A, S, L, NS, ledger_code, event_ref, comment, sub_e_code) "
            .    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )",
          plug => [
            $self->get('type'),
            $self->get('date'),
            $counter,
            $lotbalobj->get('account'),
            $lotbalobj->get('subAccount'),
            $lot,
            (0.0 - $self->get('shares')),
            $self->get('ledgerCode'),
            $self->get('id'),
            $self->get('comment'),
            $lotbalobj->get('subEntity'),
          ]
        );
        if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
        else
        {
          $self->{dbHandle}->write(
            sql => "INSERT INTO sec_event_tb (event_code, trans_date, counter, sub_e_code, A, S, L, NS, ledger_code, event_ref, comment, buy_date) "
              .                      "VALUES (?,          ?,          ?,       ?,          ?, ?, ?, ?,  ?,           ?,         ?,       (SELECT MIN(buy_date) FROM sec_event_tb WHERE l = ?) )",
            plug => [
              $self->get('type'),
              $self->get('date'),
              $counter,
              $self->get('subEntity'),
              $self->get('account2'),
              $lotbalobj->get('subAccount'),
              $newLot,
              $self->get('shares'),
              $self->get('ledgerCode'),
              $self->get('id'),
              $self->get('comment'),
              $lot,
            ]
          );
          if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
          return -1;
        }
      }
    }
    return -1;
  }
 #split the lots into individual events
  foreach my $l (@lots)
  {
    my ($lot, $shares) = split('=',$l);
    my $event = Portal::Accounting::Investments::Objects::StockTransfer->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
    $event->populate(
      lot => $lot,
      shares => $shares,
      date => $self->get('date'),
      account => $self->get('account'),
      account2 => $self->get('account2'),
      subEntity => $self->get('subEntity'),
      comment => $self->get('comment'),
      stock => $self->get('stock'),
    );
    if ($event->error) { $self->error($event->errorMessage); last; }
    $event->insert;
    if ($event->error) { $self->error($event->errorMessage); last; }
    push(@events, $event);
    $totalShares += $shares;
  }
  if (!$self->error && $self->get('shares') != $totalShares)
  {
    $self->error("<div>shares do not match<br>".$self->get('shares')." != $totalShares</div>");
  }

  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return @events;
}

=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $lotNode = shift;
  my $new1 = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode') );
  my $new2 = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode') );

  # calculate the transfer value and the transfer cost
  my $oldNodeShares = 0;
  my $oldNodeBalance = 0.00;
  my $newCost = sprintf("%.2f", $lotNode->get('cost') );

  if ($self->get('shares') < $lotNode->get('shares') )
  {
    $newCost = sprintf("%.2f", $lotNode->get('cost') / $lotNode->get('shares') * $self->get('shares') );
    $oldNodeBalance = sprintf("%.2f", $lotNode->get('cost') - $newCost );
    $oldNodeShares = $lotNode->get('shares') - $self->get('shares');
  }

  my $transobj = Portal::Accounting::Objects::JournalEntry->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, ledgerCode => $self->get('ledgerCode'));
  $transobj->set('journal', $self->{Journal});
  $transobj->set('date', $self->get('date') );
  $transobj->addRow(
    subEntity => $lotNode->get('subEntity'),
    account => $self->get('account'),
    subAccount => $self->get('stock'),
    dorc => "false",
    amount => $newCost,
    comment => $self->get('comment'),
  );
  $transobj->addRow(
    subEntity => $self->get('subEntity'),
    account => $self->get('account2'),
    subAccount => $self->get('stock'),
    dorc => "true",
    amount => $newCost,
    comment => $self->get('comment'),
  );
  if ($transobj->error())
  {
    $self->error($transobj->errorMessage);
    return 0;
  }
  else
  {
    # insert the transaction into the database
    my $refNum = $transobj->insert();
    if ($transobj->error)
    {
      $self->error($transobj->errorMessage);
      return 0;
    }

    # create two new lot nodes, one for the new lot that is generated from the event and one for the old lot to decrease
    # its balances (shares, cost)
    $new2->populate(
      date => $self->get('date'),
      parent => $lotNode->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $self->get('subEntity'),
      account => $self->get('account2'),
      subAccount => $lotNode->get('subAccount'),
      lot => $self->get('lot2'),
      buyLot => $lotNode->get('buyLot'),
      cost => $newCost,
      shares => sprintf("%.6f", $self->get('shares')),
    );
    if ($new2->error) { $self->error($new2->errorMessage); return 0; }
    $new2->insert;
    if ($new2->error) { $self->error($new2->errorMessage); return 0; }
    $new1->populate(
      date => $self->get('date'),
      parent => $lotNode->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $lotNode->get('subEntity'),
      account => $self->get('account'),
      subAccount => $lotNode->get('subAccount'),
      lot => $lotNode->get('lot'),
      buyLot => $lotNode->get('buyLot'),
      cost => $oldNodeBalance,
      shares => sprintf("%.6f", $oldNodeShares),
    );
    $new1->insert;
    if ($new1->error) { $self->error($new1->errorMessage); return 0; }
    if ($new1->error) { $self->error($new1->errorMessage); return 0; }
    $self->SUPER::propagate($new1);
    $self->SUPER::propagate($new2);
  }
  return 1;
}

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.

=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut