#$Id: Opening.pm,v 1.7 2006/05/18 20:12:18 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

JournalEntry::Opening - Object used to hold forms for JournalEntry::Opening and manipulate the opening balances for the books

=head1 SYNOPSIS

  use JournalEntry::Opening;
  my $transobj = JournalEntry::Opening->new(langObj =>);
  if ($transobj->error())
  {
    die $transobj->errorMessage();
  }

  my $form = $transobj->printForm();

=head1 DESCRIPTION

JournalEntry::Opening is a class that stores data, methods, and forms for a
double entry view used to enter opening balances for a ledger.  It derives
from JournalEntry so it provides methods to write to the database or update
an existing record in the database.

=head1 Exported FUNCTIONS

=cut

package Portal::Accounting::Objects::JournalEntry::Opening;

use strict;
use Portal::Accounting::Objects::JournalEntry::Base;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::SubEntity;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::JournalEntry::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.7 $';

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => "", @_);
  my $act = Portal::Accounting::Objects::Account->new(@_);
  my $sact = Portal::Accounting::Objects::SubAccount->new(@_);
  my $se = Portal::Accounting::Objects::SubEntity->new(@_);
  if ($act->error) { $self->error($act->error); }
  if ($sact->error) { $self->error($sact->error); }
  if ($se->error) { $self->error($se->error); }
  my $l = $self->{langObj};
  #use form object to get calculator, date and some non repeating fields
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "JournalEntry" },
    "command" => { -Type => "hidden", -Value => "opening" },
    "id" => { -Type => "hidden", -Value => -1, -DataType => "int", },
    "view" => { -Type => "hidden", -Value => "O", -DataType => "string" },
    "journal" => { -Type => "hidden", -Value => "O", -DataType => "string", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "date" => { -Type => "text", size => "7", -Label => $l->map("Date"), class => 'date',
      -Value => $self->{ledger}->preOpenBooksDate, -ReadOnly => 1,
    },
    "payee" => { -Type => "searchBox", -Label => $l->map("Description"),  -shareDATA => "PAYEES",
      -shareData => "PAYEES", -isAA => 1, class => 'ledgerBorderColor',
      -onload => 'init(document.main.id.value);',
    },
    "memo" => { -Type => "hidden", -DataType => "string", },
    "subEntity_0" => { -Type => "hidden", -Label => $l->map("Sub Entity"), },
    "subEntity_Display0" => { -Label => $l->map("Sub Entity"), class => 'subEntity',
      -Type => "text", -DataType => "int", class => 'subEntity',
      -onload => <<"JS",
  result = searchBox(document.main.subEntity_Display0,
    document.main.subEntity_0,
    SUBENTITIES,
    false, true, true, false,
    '',
    'if (! subEntityCheck(document.main.subEntity_0)) { document.main.subEntity_Display0.select(); }'
  );
JS
    },
    "account_0" => { -Type => "hidden", -Label => $l->map("Account"), },
    "account_Display0" => { -Label => $l->map("Account"),
      -Type => "text", -DataType => "int", class => "account",
      -onload => <<"JS",
  result = searchBox(
    document.main.account_Display0
    , document.main.account_0
    , EQUITYACCOUNTS
    , false, true, true, true
    , ''
    , 'if (!equityAccountCheck(0)) { alert(langObj["mustChoose"]); } '
  );
JS
    },
    "subAccount_0" => { -Type => "hidden"},
    "subAccount_Display0" => { -Type => "hidden" },
    "remember" => { -Type => "checkbox", style => "display: none;" },
  };
  $self->{css}->{main} .= <<"ENDOF";
    .row { clear: both; }
    .heading { font-style: bold; }
    .account { border: none; width: 10em; float: left; margin:0; margin-right: 1px; }
    .subAccount { border: none; width: 10em; float: left; margin: 0; margin-right: 1px; margin-bottom: 1px;}
    .subEntity { border: none; width: 3em; float: leftt; margin: 0; margin-right: 1px; text-align: center; }
    .amount { border: none; width: 7em; text-align:right; float: left; margin: 0; margin-right: 1px;}
    .subEntity { border: none; float: left; margin: 0; margin-right: 1px;}
    .comment { border: none; width: 20em; float: left; margin: 0; margin-right: 1px;}
    .delete {float: left; text-align: center; }

    .oAccount { float: left; margin: .5em; }
    .oSubAccount { float: left;  margin: .5em; }
    .description { clear: float; float: left;  margin: .5em; }
    .memo { clear: float; float: left; margin: .5em; }
    .remember { float: left;  margin: .5em; }
    .oSubEntity { float: right;  margin: .5em; }
    .oAmount { float: right;  margin: .5em; }
    .date { float: right;  margin: .5em; }
    .spacer { clear: both; }
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<div id='O' style="display: none;" class="row">
  <input type="text" id="subEntity_Display" name="subEntity_Display" value="" class="subEntity" onChange="clearMe(this);" />
  <div style="float:left;">
    <input type="text" id="account_Display" name="account_Display" value="" class="account" onChange="clearMe(this); " />
  </div>
  <input type="text" id="subAccount_Display" name="subAccount_Display" value="" class="subAccount" onChange="clearMe(this); " />
  <input type="text" id="debit_" name="debit_" value="" class="amount" onChange="calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); debitCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  <input type="text" id="credit_" name="credit_" value="" class="amount" onChange="calculateFormula(this, false); clearMe(this); this.value = floatify(this.value,2); creditCheck(this); if (! firstRowCheck(this)) { lastRowCheck(); }" />
  <input type="text" id="comment_" name="comment_" value="" class="comment" maxlength="255" />
  <input class="delete" type="button" name="delete_" value="X" onclick="return deleteRow(this);" style="border: none;" />

  <input type="hidden" name="cleared_" value="_">

  <input type="text" size="3" name="account_"    class="debug" />
  <input type="text" size="3" name="subAccount_" class="debug" />
  <input type="text" size="3" name="subEntity_"  class="debug" />

</div>

$self->{requiredDivs}

<div id="mainDiv">
  <form name="main" target="_blank" method="post" action="">

    <div class="primary">
      #FIELD=view#
      #LABEL=date# #FIELD=date#&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
      #FIELD=ledgerCode#
      #FIELD=journal#
      #LABEL=payee# #FIELD=payee#
      #FIELD=remember#
      #FIELD=memo#
      <div style="margin: 1em;" class="spacer"></div>

      <div style="padding: .5em;" class="tertiary">
        <div style="float: left; width: 50em; ">
          <div class="heading">
            <div class="subEntity"><span title="#LANG=SubEntity#">#LANG=SE#</span></div>
            <div class="account">#LANG=Account#</div>
            <div class="amount" style="text-align: center;">#LANG=Debit#</div>
            <div class="amount" style="text-align: center;">#LANG=Credit#</div>
            <div class="comment">#LANG=Comment#</div>
            <div class="delete" title="#LANG=Delete#"></div>
          </div>

          <div id="row_0" style="margin-top: 1em; clear: both;" >
            <div style="float: left;">
              #FIELD=subEntity_Display0#
              #FIELD=account_Display0#
              #FIELD=subAccount_Display0#
            </div>
            <div >
              <input type="text" id="debit_0" name="debit_" value="" class="amount" onfocus="this.form.comment_0.select(); " />
              <input type="text" id="credit_0" name="credit_" value="" class="amount"  onfocus="this.form.comment_0.select(); " />
              <input type="text" id="comment_0" name="comment_" value="" class="comment" maxlength="255" />
            </div>

            <input type="hidden" name="cleared_0" value="_">

            <input type="text" size="3" name="account_0"    class="debug" />
            <input type="text" size="3" name="subAccount_0" class="debug" />
            <input type="text" size="3" name="subEntity_0"  class="debug" />
          </div>
        </div>
        #LANG=equityNote#
        <div class="spacer" style="width: 50em; "></div>
      </div>

      <div class="heading spacer">
        <div class="subEntity"><span title="#LANG=SubEntity#">#LANG=SE#</span></div>
        <div class="account">#LANG=Account#</div>
        <div class="subAccount">&nbsp;#LANG=Sub Account#</div>
        <div class="amount" style="text-align: center;">#LANG=Debit#</div>
        <div class="amount" style="text-align: center;">#LANG=Credit#</div>
        <div class="comment">#LANG=Comment#</div>
        <div class="delete" title="#LANG=Delete#"></div>
        <div style="clear: both;"></div>
      </div>

      <div class="spacer" id="insertRowHere"></div>

      <div style="clear: both; padding-top: 1em; ">
        <input style="float:left;" name="submitDataButton" onclick="return submitWrapper();" value="#LANG=Insert#" type="button">
        <input style="float:left;" onclick="javascript: return addRow(this.form.view.value,'insertRowHere');" name="add" value="#LANG=Add Row#" type="button">
        <input style="float:right;" type="button" value="#LANG=Reset#" onclick="return resetForm();" />
        <div style="float:right;" style="text-align:right;">
          <a title="#LANG=editHelp#">#</a>
          #FIELD=id#<span id="refNumSpan">&nbsp;</span>
          <input type="hidden" name="editID" />
        </div>
      </div>

      <div class="spacer"></div>
    </div>

$self->{optionsDiv}

  </form>
</div>

<div class="tertiary" style="padding: .5em;">
  <span style="font-weight: bold; border-bottom-style: solid; border-weight: thin;">#LANG=Help#</span>
  #LANG=openingHelp#
  #LANG=otherTips#
</div>
ENDOF
  $self->{javascript}->{main} .= $act->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $sact->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  $self->{javascript}->{main} .= $se->javascriptOverhead($self->{data}->{main}->{ledgerCode}->{-Value});
  if ($act->error) { $self->error($act->errorMessage); }
  if ($sact->error) { $self->error($sact->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }

  $self->{profile}->{main} = {
    required => [  ],
    optional => [  ],
    constraints => {
    },
  };
  $self->{defaultForm} = "main";

  return $self;
}

=item boolean isValid

  This routine will always return true because the double entry
  object is just a placeholder for the screen used to edit and
  enter a double entry transaction.

=cut
sub isValid
{
  return 1;
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

JT Moree

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
