# Fed941.pm - The Object Class that provides a Fed941 Object
#$Id: Fed941.pm,v 1.10.6.1 2006/07/14 20:05:59 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Payroll::Fed941;
use strict;
use Portal::Accounting::Objects::BaseL;
use Business::Payroll::US::Medicare;
use Business::Payroll::US::FICA;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.10.6.1 $';

=head1 NAME

Fed941 - Object used to build a Fed941 Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Fed941;
  my $Fed941Obj = Portal::Accounting::Objects::Fed941->new(   );
  if ($Fed941Obj->error())
  {
    die $Fed941Obj->errorMessage();
  }

=head1 DESCRIPTION

Fed941 holds forms for dealing with Federal 941 payroll reports.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Fed941.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  $self->{FICA} = Business::Payroll::US::FICA->new;
  $self->{Medicare} = Business::Payroll::US::Medicare->new;

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  my @date = Today();
  my $quarter = 4;
  if ($date[1] > 3) { $quarter = 1; }
  if ($date[1] > 6) { $quarter = 2; }
  if ($date[1] > 9) { $quarter = 3; }
  my $l = $self->{langObj};
  $self->{defaultForm} = "form";
  $self->{data}->{main} = {
    "year" => { -Type => "text", -ReadOnly => 1, -DataType => "integer", },
    "quarter" => { -Type => "text", -ReadOnly => 1, -DataType => "string", },
    "name" => { -Type => "text", -ReadOnly => 1, -DataType => "string", },
    "1" => { -Type => "text", -ReadOnly => 1, },
    "2" => { -Type => "text", -ReadOnly => 1, },
    "3" => { -Type => "text", -ReadOnly => 1, },
    "4" => { -Type => "text", -ReadOnly => 1, -Value => " "},
    "5a1" => { -Type => "text", -ReadOnly => 1, },
    "5a2" => { -Type => "text", -ReadOnly => 1, },
    "5a3" => { -Type => "text", -ReadOnly => 1, },
    "5b1" => { -Type => "text", -ReadOnly => 1, },
    "5b2" => { -Type => "text", -ReadOnly => 1, },
    "5b3" => { -Type => "text", -ReadOnly => 1, },
    "5c1" => { -Type => "text", -ReadOnly => 1, },
    "5c2" => { -Type => "text", -ReadOnly => 1, },
    "5c3" => { -Type => "text", -ReadOnly => 1, },
    "5d" => { -Type => "text", -ReadOnly => 1, },
    "6" => { -Type => "text", -ReadOnly => 1, },
    "6a" => { -Type => "text", -ReadOnly => 1, -Value => 0},
    "7a" => { -Type => "text", -ReadOnly => 1, },
    "sickPay" => { -Type => "text", -ReadOnly => 1, -DataType => "string", },  #7b
    "lifeInsur" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }, #7c
    "incTax" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }, #7d
    "ssMedicare" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }, #7e
    "specialFed" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }, #7f
    "specialSS" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }, #7g
    "7h" => { -Type => "text", -ReadOnly => 1, },
    "8" => { -Type => "text", -ReadOnly => 1, },
    "9" => { -Type => "text", -ReadOnly => 1, },  #aeic
    "10" => { -Type => "text", -ReadOnly => 1, },
    "11" => { -Type => "text", -ReadOnly => 1, },
    "12" => { -Type => "text", -ReadOnly => 1, },
    "13" => { -Type => "text", -ReadOnly => 1, },
    "14" => { -Type => "text", -ReadOnly => 1, },
    "15_1" => { -Type => "text", -ReadOnly => 1, },
    "15_2" => { -Type => "text", -ReadOnly => 1, },
    "15_3" => { -Type => "text", -ReadOnly => 1, },
    "15" => { -Type => "text", -ReadOnly => 1, },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll" },
    "command" => { -Type => "hidden", -Value => "report" },
    "report" => { -Type => "hidden", -Value => "Fed941" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "cap" => { -Type => "text", -ReadOnly => 1, -Value => 0 }, #default values
    "stateid" => { -Type => "text", -ReadOnly => 1, -Value => '' },
    "fedid" => { -Type => "text", -ReadOnly => 1, -Value => '' },
  };
  $self->{data}->{form} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll::Report" },
    "command" => { -Type => "hidden", -Value => "report" },
    "report" => { -Type => "hidden", -Value => "Fed941" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "name" => { -Type => "text", -DataType => "string", -Label => $l->map('Subtitle'), },
    "year" => {-Type => "text", size => 4, -Label => $l->map('Year'),
      -DataType => "integer", -Value => $date[0],
    },
    "quarter" => {-Type => "select", -Label => $l->map('Quarter'),
      -DataType => "string", -Value => $quarter,
      -Options => { names => [qw(1 2 3 4)], values => [qw(1 2 3 4)] },
    },
    "sickPay" => {-Type => "calculator", size => 10, -Label => $l->map('sickPay'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "lifeInsur" => {-Type => "calculator", size => 10, -Label => $l->map('lifeInsur'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "incTax" => {-Type => "calculator", size => 10, -Label => $l->map('incTax'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "ssMedicare" => {-Type => "calculator", size => 10, -Label => $l->map('ssMedicare'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "specialFed" => {-Type => "calculator", size => 10, -Label => $l->map('specialFed'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
    "specialSS" => {-Type => "calculator", size => 10, -Label => $l->map('specialSS'),
      -DataType => "float", -WidgetOptions => { undoButton => 0,}
    },
  };
   $self->{css}->{main} = <<"ENDOF";
.fed941 {
  background-color: #cccccc;
  text-align: left;
  vertical-align: top;
}
.number {
  text-align: right;
}
.field {
  background-color: #ffffff;
  text-align: right;
  vertical-align: top;
  width: 5em;
}
ENDOF
   $self->{template}->{form} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=report#
  <table class="primary" cellpadding="5">
    <tr>
      <td colspan="2">
        #LABEL=name# #FI=name#
      </td>
    </tr>
    <tr>
      <td colspan="2">
        #LABEL=year#
        #FI=year#
        #LABEL=quarter#
        #FI=quarter#
      </td>
    </tr>
    <tr><td class="number">#LABEL=sickPay#</td><td>#FI=sickPay#</td></tr>
    <tr><td class="number">#LABEL=lifeInsur#</td><td>#FI=lifeInsur#</td></tr>
    <tr><td class="number">#LABEL=incTax#</td><td>#FI=incTax#</td></tr>
    <tr><td class="number">#LABEL=ssMedicare#</td><td>#FI=ssMedicare#</td></tr>
    <tr><td class="number">#LABEL=specialFed#</td><td>#FI=specialFed#</td></tr>
    <tr><td class="number">#LABEL=specialSS#</td><td>#FI=specialSS#</td></tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
   $self->{template}->{main} = <<"ENDOF";
  #LANG=Federal Tax ID#: #FIELD=fedid#
  <table width="100%" cellpadding="5" class="fed941">
    <tr>
      <th colspan="3">
        Part 1: #LANG=answerThese#
      </td>
    </tr>
    <tr>
      <td class="number">1</td>
      <td class="lang">#LANG=numEmployees#</td>
      <td class="field">#FIELD=1#</td>
    </tr>
    <tr>
      <td class="number">2</td>
      <td class="lang">#LANG=wagesTipsOther#</td>
      <td class="field">#FIELD=2#</td>
    </tr>
    <tr>
      <td class="number">3</td>
      <td class="lang">#LANG=TotalIncTax#</td>
      <td class="field">#FIELD=3#</td>
    </tr>
    <tr>
      <td class="number">4</td>
      <td class="lang">#LANG=NoSSMedTax#</td>
      <td align="left"><span class="field">[#FIELD=4#]<span></td>
    </tr>
    <tr>
      <td class="number">5</td>
      <td class="lang">#LANG=SSMedicareWagesTips#</td>
      <td class="field">&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td class="lang">
        <table width="100%">
          <tr>
            <td>5a #LANG=SSWages#</td>
            <td class="field">#FIELD=5a1#</td>
            <td align="right">x #FIELD=5a3# =</td>
            <td class="field">#FIELD=5a2#</td>
          </tr>
          <tr>
            <td>5b #LANG=SSTips#</td>
            <td class="field">#FIELD=5b1#</td>
            <td align="right">x #FIELD=5b3# =</td>
            <td class="field">#FIELD=5b2#</td>
          </tr>
          <tr>
            <td>5c #LANG=MedicareWagesTips#</td>
            <td class="field">#FIELD=5c1#</td>
            <td align="right">x #FIELD=5c3# =</td>
            <td class="field">#FIELD=5c2#</td>
          </tr>
        </table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td class="lang">5d #LANG=TotalSSMedicare#</td>
      <td class="field">#FIELD=5d#</td>
    </tr>
    <tr>
      <td class="number">6</td>
      <td class="lang">#LANG=Total taxes before adjustments#</td>
      <td class="field">#FIELD=6#</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td class="lang">#LANG=Actual taxes withheld# #LABEL=7a#</td>
      <td class="field">#FIELD=6a#</td>
    </tr>
    <tr>
      <td class="number">7</td>
      <td colspan="2">
        #LANG=Tax adjustments#
      </td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7a #LANG=fractions#</td>
          <td class="field">#FIELD=7a#</td>
        </tr></table>
      </td>
      <td>&nbsp;#LABEL=7a#</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7b #LANG=sickPay#</td>
          <td class="field">&nbsp;#FIELD=sickPay#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7c #LANG=lifeInsur#</td>
          <td class="field">&nbsp;#FIELD=lifeInsur#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7d #LANG=incTax# (#LANG=941c#)</td>
          <td class="field">#FIELD=incTax#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7e #LANG=ssMedicare# (#LANG=941c#)</td>
          <td class="field">#FIELD=ssMedicare#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7f #LANG=specialFed# (#LANG=reserved use#)</td>
          <td class="field">#FIELD=specialFed#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td>
        <table width="100%"><tr>
          <td class="lang">7g #LANG=specialSS# (#LANG=reserved use#)</td>
          <td class="field">#FIELD=specialSS#</td>
        </tr></table>
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td class="number">&nbsp;</td>
      <td class="lang">
        <table width="100%"><tr>
          <td class="lang">7h #LANG=Total adjustments#</td>
        </tr></table>
      </td>
      <td class="field">#FIELD=7h#</td>
    </tr>
    <tr>
      <td class="number">8</td>
      <td class="lang">#LANG=TaxesAfter#</td>
      <td class="field">#FIELD=8#</td>
    </tr>
    <tr>
      <td class="number">9</td>
      <td class="lang">#LANG=aeic#</td>
      <td class="field">#FIELD=9#</td>
    </tr>
    <tr>
      <td class="number">10</td>
      <td class="lang">#LANG=totalAfterAEIC#</td>
      <td class="field">#FIELD=10#</td>
    </tr>
    <tr>
      <td class="number">11</td>
      <td class="lang">#LANG=totalDeposits#</td>
      <td class="field">#FIELD=11#</td>
    </tr>
    <tr>
      <td class="number">12</td>
      <td class="lang">#LANG=Balance Due#</td>
      <td class="field">#FIELD=12#</td>
    </tr>
    <tr>
      <td class="number">13</td>
      <td class="lang">#LANG=Overpayment#</td>
      <td class="field">#FIELD=13#</td>
    </tr>
    <tr>
      <th colspan="3">
        Part 2: #LANG=TellUsDeposits#
      </td>
    </tr>
    <tr>
      <td class="number">14</td>
      <td class="lang" colspan="2"><span class="field">[ ][ ]</span> #LANG=stateAbbreviation#</td>
    </tr>
    <tr>
      <td class="number">15</td>
      <td class="lang" colspan="2">
        <span class="field">[ ]</span> <b>#LANG=line10#.</b> #LANG=ToPart3#. <br>
        <span class="field">[ ]</span> <b>#LANG=monthlyDepositor#.</b>
        #LANG=ToPart3#.<br>
        <table>
          <tr><td><b>#LANG=Tax Liability#:</b></td><td><b>#LANG=Month# 1</b></td><td class="field">#FIELD=15_1#</td><td>&nbsp;</td></tr>
          <tr><td>&nbsp;</td><td><b>#LANG=Month# 2</b></td><td class="field">#FIELD=15_2#</td><td>&nbsp;</td></tr>
          <tr><td>&nbsp;</td><td><b>#LANG=Month# 3</b></td><td class="field">#FIELD=15_3#</td><td>&nbsp;</td></tr>
          <tr><td>&nbsp;</td><td><b>#LANG=Total#</b></td><td class="field">#FIELD=15#</td><td>#LANG=mustEqual10# #LABEL=15#</td></tr>
        </table>
      </td>
    </tr>
  </table>
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( year quarter ) ],
    optional => [ qw( name sickPay lifeInsur incTax ssMedicare specialFed specialSS aeic ) ],
    constraints => {
    },
  };
  $self->{profile}->{form} = {
    required => [ qw( year quarter report ) ],
    optional => [ qw( name sickPay lifeInsur incTax ssMedicare specialFed specialSS aeic ) ],
    constraints => {
      year => qr/^\d{4}$/,
      quarter => qr/^[1-4]$/,
      sickPay => qr/^\d+\.?\d*$/,
      specialSS => qr/^\d+\.?\d*$/,
      specialFed => qr/^\d+\.?\d*$/,
      ssMedicare => qr/^\d+\.?\d*$/,
      incTax => qr/^\d+\.?\d*$/,
      lifeInsur => qr/^\d+\.?\d*$/,
      aeic => qr/^\d+\.?\d*$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  Gathers data from database for this report.

=cut
sub select
{
  my $self = shift;
  my $year = $self->get('year');
  my $q = $self->get('quarter');
  my $to = '0331';
  my $from = '0101';
  my $l = $self->get('ledgerCode');
  my $previous = 0;
  my $qStart;
  my $total = 0;
  if ($q == 4)
  {
    $from = '1001';
    $to = '1231';
    $previous = 1;
  }
  elsif ($q == 3)
  {
    $from = '0701';
    $to = '0930';
    $previous = 1;
  }
  elsif ($q == 2)
  {
    $from = '0401';
    $to = '0630';
    $previous = 1;
  }
  $qStart = $from;
  #check rates in perl payroll modules and set in object!!!
  my ($mRate, $cap) = $self->{Medicare}->rateCap($year.$from);
  my ($ssRate, $cap) = $self->{FICA}->rateCap($year.$from);
  $mRate *= 2.0;
  $ssRate *= 2.0;
  $self->set('5a3', $ssRate);
  $self->set('5b3', $ssRate);
  $self->set('5c3', $mRate);
  $self->set('cap', $cap);

  #count number of employees
  $self->{LIST}[0] = <<"ENDOF";
  SELECT COUNT(DISTINCT(d.sub_account_num))
  FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
  WHERE d.ledger_code = ?
    AND s.type = 120
    AND s.sub_acc_number = d.sub_account_num
    AND d.account_num = a.number
    AND (
      a.type = 55
      OR a.type = 24
      OR a.type = 26
      OR a.type = 27
    )
    AND d.trans_date BETWEEN ? AND ?
ENDOF
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[0], plug => [ $l , $year.$from, $year.$to ]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my @data = $sth->fetchrow_array)
  {
    $self->set('1', $data[0]);
  }

  #fitw, ss, medicare, aeic
  for (my $x = 1; $x <= 3; $x++)
  {
    my $month = ($q - 1) * 3 + $x;
    my $month2 = $month + 1;
    my $year2 = $year;
    if ($month2 > 12)
    {
      $month2 = 1;
      $year2 = $year + 1;
    }
    my $one = '01';
    $self->{LIST}[$x] = <<"ENDOF";
    SELECT a.type, SUM(d.d_amount) AS amount
    FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
    WHERE d.ledger_code = ?
      AND (
        (d.sub_account_num = 10 AND s.sub_acc_number = d.sub_account_num)
        OR (s.type = 120 AND s.sub_acc_number = d.sub_account_num)
      )
      AND d.account_num = a.number
      AND (
        a.type = 24
        OR a.type = 26
        OR a.type = 27
        OR a.type = 28
      )
      AND d.trans_date >= ?
      AND d.trans_date < ?
    GROUP BY a.type
ENDOF
    $total = 0;
    my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[$x], plug => [ $l, $year.sprintf('%02d',$month).$one, $year2.sprintf('%02d',$month2).$one ]);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
    while (my $data = $sth->fetchrow_hashref)
    {
      if ($data->{type} eq '24')
      {
        $self->set('3', sprintf("%.02f",$self->get('3') + $data->{amount}) );
      }
      elsif ($data->{type} eq '26' || $data->{type} eq '27')
      {
        $self->set('6a', sprintf("%.02f",$self->get('6a') + $data->{amount}) );
      }
      elsif ($data->{type} eq '28')
      {
        $self->set('9', sprintf("%.02f",$self->get('9') + $data->{amount}) );
      }
      $self->{"TYPE_".$x."_".$data->{type}} = $data->{amount};
      $self->{"TYPE_".$data->{type}} += $data->{amount};
      $total += $data->{amount};
    }
    #set 15 based on total
    $self->set('15_'.$x, sprintf("%.02f",$total) );
    $self->set('15', sprintf("%.02f",$self->get('15') + $total));
  }

  #salaries by employee
  $self->{LIST}[4] = <<"ENDOF";
  SELECT s.sub_acc_name AS name, SUM(d.d_amount) AS amount
  FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
  WHERE d.ledger_code = ?
    AND s.type = 120
    AND s.sub_acc_number = d.sub_account_num
    AND d.account_num = a.number
    AND a.type = 55
    AND d.trans_date BETWEEN ? AND ?
  GROUP BY s.sub_acc_name
  ORDER BY s.sub_acc_name
ENDOF
  $total = 0;
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[4], plug => [ $l, $year.$from, $year.$to ]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    $self->{EMPLOYEES_PREVIOUS}->{$data->{name}} = 0;
    $self->{EMPLOYEES}->{$data->{name}} = $data->{amount};
    $total += $data->{amount};
  }
  $self->set('2',sprintf("%.02f",$total));

  #ytd numbers
  my $one = '0101';
  $self->{LIST}[5] = <<"ENDOF";
  SELECT s.sub_acc_name AS name, SUM(d.d_amount) AS amount
  FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
  WHERE d.ledger_code = ?
    AND s.type = 120
    AND s.sub_acc_number = d.sub_account_num
    AND d.account_num = a.number
    AND a.type = 55
    AND d.trans_date >= ? AND d.trans_date < ?
  GROUP BY s.sub_acc_name
ENDOF
  if ($previous)
  {
    my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[5], plug => [ $l, $year.$one, $year.$from ]);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
    while (my $data = $sth->fetchrow_hashref)
    {
      $self->{EMPLOYEES_PREVIOUS}->{$data->{name}} = $data->{amount};
    }
  }
  $total = 0;
  foreach my $e (keys %{$self->{EMPLOYEES}})
  {
    if ($self->{EMPLOYEES_PREVIOUS}->{$e} < $cap)
    {
      if ($self->{EMPLOYEES}->{$e} + $self->{EMPLOYEES_PREVIOUS}->{$e} <= $cap)
      {
        $total += $self->{EMPLOYEES}->{$e};
      }
      else
      {
        $total += $cap - $self->{EMPLOYEES_PREVIOUS}->{$e};  #add employee less than cap onto total
      }
    }
  }
  $total = sprintf("%.02f",$total);
  my $ss = sprintf("%.02f", $total * $ssRate);
  my $m = sprintf("%.02f", $self->get('2') * $mRate);
  $self->set('5a1',$total);
  $self->set('5b1',0);
  $self->set('5c1',$self->get('2'));
  $self->set('5a2', $ss);
  $self->set('5b2',0);
  $self->set('5c2', $m );
  $self->set('5d', sprintf("%.02f", $ss + 0 + $m) );
  if ($self->get('5d') == 0) { $self->set('4', "X"); }
  $self->set('6', sprintf("%.02f",$self->get('3') + $ss + 0 + $m) );
  $self->set('6a', sprintf("%.02f", $self->get('3') + $self->get('6a')) );
  #check internal cache of selected data for ss and medicare to get fractions of cents
  my $fractions = sprintf("%.02f", $self->get('6a') - $self->get('6') );
  $self->set('7a', sprintf("%.02f",$fractions));
  $total = $self->get('7a');
  $total += $self->get('sickPay');
  $total += $self->get('lifeInsur');
  $total += $self->get('incTax');
  $total += $self->get('ssMedicare');
  $total += $self->get('specialFed');
  $total += $self->get('specialSS');
  $self->set('7h', sprintf("%.02f",$total));
  $self->set('8', sprintf("%.02f",$total + $self->get('6')) );
  $self->set('10', sprintf("%.02f",$self->get('8') - $self->get('9')) );
  if ($self->get('10') != $self->get('15'))
  {
    $self->{data}->{main}->{'15'}->{-Label} = "<font color=\"red\">#LANG=Does not match#</font>";
  }
  if ($self->get('7a') > 0 && $self->get('7a') / $self->get('6') > 0.05 )
  {
    $self->{data}->{main}->{'7a'}->{-Label} = "<font color=\"red\">#LANG=Excessive Difference# #LANG=Warning#</font>";
  }
  #monthly deposits within a year's time frame
  $self->{LIST}[6] = <<"ENDOF";
  SELECT -SUM(d.d_amount) AS amount
  FROM debitcredit_tb AS d, account_tb AS a
  WHERE d.ledger_code = ?
    AND d.sub_account_num = $q
    AND d.account_num = a.number
    AND (
      a.type = 24
      OR a.type = 26
      OR a.type = 27
    )
    AND d.trans_date >= ?
ENDOF
  $self->{LIST}[6] .= " AND d.trans_date < ?";
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[6], plug => [ $l, $year.$qStart, ($year + 1) . $qStart]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    $self->set('11', $data->{amount});
  }

  if ($self->get('10') - $self->get('11') > 0)
  {
    $self->set('12', sprintf("%.02f",$self->get('10') - $self->get('11')) );
  }
  elsif ($self->get('11') - $self->get('10') > 0)
  {
    $self->set('13', sprintf("%.02f",$self->get('11') - $self->get('10')) );
  }

  $self->{LIST}[7] = <<"ENDOF";
  SELECT fedid, stateid
  FROM ledger_tb
  WHERE code = ?
ENDOF
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[7], plug => [ $l ] );
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    $self->set('fedid', $data->{fedid});
    $self->set('stateid', $data->{stateid});
  }
  return 1;
}

sub printReport
{
  my $self = shift;
  my $name = $self->get('name');
  if ($name ne '') { $name .= "<br />\n"; }
  my $r = "<center>".$name;
  $r .= $self->get('year')." ".$self->langObj->map('Quarter')." ".$self->get('quarter')."<br /></center>\n";
  my %f = $self->printForm(form => 'main');
  if ($self->{formObj}->error) { $self->error($self->{formObj}->errorMessage); }
  else
  {
    $f{body} = $r.$f{body} . $self->debugString;
  }
  $f{css} = $self->{css}->{main};
  return %f;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
