# Fed941Deposit.pm - The Object Class that provides a Fed941Deposit Object
#$Id: Fed941Deposit.pm,v 1.5 2006/03/29 21:03:44 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Payroll::Fed941Deposit;
use strict;
use Portal::Accounting::Objects::BaseL;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.5 $';

=head1 NAME

Fed941Deposit - Object used to build a Fed941Deposit Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Fed941Deposit;
  my $Fed941DepositObj = Portal::Accounting::Objects::Fed941Deposit->new(   );
  if ($Fed941DepositObj->error())
  {
    die $Fed941DepositObj->errorMessage();
  }

=head1 DESCRIPTION

Fed941Deposit holds forms for dealing with Federal 941 payroll reports.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Fed941Deposit.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  $self->{new} = 1;
  my @date = Today();
  my $year = $date[0];
  my $lastMonth = $date[1] - 1;
  if ($lastMonth <= 0) { $lastMonth = 12 ; $year = $date[0] - 1; }
  my $l = $self->{langObj};
  my $act = Portal::Accounting::Objects::Account->new(@_);
  $act->set('type', 24);
  my $t24 = $act->selectOptionsByType();
  $act->set('type', 26);
  my $t26 = $act->selectOptionsByType();
  $act->set('type', 27);
  my $t27 = $act->selectOptionsByType();
  $act->set('type', 28);
  my $t28 = $act->selectOptionsByType();

  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => 'Accounting' },
    "state" => { -Type => "hidden", -Value => 'JournalEntry' },
    "command" => { -Type => "hidden", -Value => 'federalDeposit' },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
    "year" => { -Type => "text", -DataType => "integer", size => 4,
      -Value => $year, -Label => $l->map('Year'),
    },
    "month" => { -Type => "select", -DataType => "integer",
      -Options => { names => [ qw( 1 2 3 4 5 6 7 8 9 10 11 12) ],
        values => [qw( 1 2 3 4 5 6 7 8 9 10 11 12)]
      }, -Label => $l->map('Month'), -Value => $lastMonth,
    },
    "quarter" => { -Type => "hidden", },
    "total" => { -Type => "hidden", },
    "24" => { -DataType => "integer", -Type => "select", -Options => $t24, -Label => $l->map('Federal Inc Tax Liability')." ".$l->map('Account'), },
    "26" => { -DataType => "integer", -Type => "select", -Options => $t26, -Label => $l->map('FICA Liability')." ".$l->map('Account'), },
    "27" => { -DataType => "integer", -Type => "select", -Options => $t27, -Label => $l->map('Medicare Liability')." ".$l->map('Account'), },
    "28" => { -DataType => "integer", -Type => "select", -Options => $t28, -Label => $l->map('Advanced Earned Income Credit')." ".$l->map('Account'), },
  };
   $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  <table class="primary" cellpadding="5">
    <tr>
      <td>
        #LABEL=year#
        #FI=year#
        #LABEL=month#
        #FI=month#
      </td>
    </tr>
    <tr><td>
      #LABEL=24# #FI=24#
    </tr></td>
    <tr><td>
      #LABEL=26# #FI=26#
    </tr></td>
    <tr><td>
      #LABEL=27# #FI=27#
    </tr></td>
    <tr><td>
      #LABEL=28# #FI=28#
    </tr></td>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( year month ) ],
    optional => [  ],
    constraints => {
      year => qr/^\d{4}$/,
      month => qr/^[1-9][0-2]*$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  my $self = shift;
  return $self->{new};
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  Gathers data from database for this report.

=cut
sub select
{
  my $self = shift;
  $self->{new} = 0;
  my $year = $self->get('year');
  my $month = $self->get('month');
  my $l = $self->get('ledgerCode');

  #fitw, ss, medicare, aeic
  my $month2 = $month + 1;
  my $year2 = $year;
  my $q = 1;
  if ($month > 3) { $q = 2; }
  if ($month > 6) { $q = 3; }
  if ($month > 9) { $q = 4; }
  $self->set('quarter', $q);
  if ($month2 > 12)
  {
    $month2 = 1;
    $year2 = $year + 1;
  }
  my $one = '01';
  $self->{LIST}[0] = <<"ENDOF";
    SELECT a.type, SUM(d.d_amount) AS amount
    FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
    WHERE d.ledger_code = ?
      AND (
        (d.sub_account_num = 10 AND s.sub_acc_number = d.sub_account_num)
        OR (s.type = 120 AND s.sub_acc_number = d.sub_account_num)
      )
      AND d.account_num = a.number
      AND (
        a.type = 24
        OR a.type = 26
        OR a.type = 27
        OR a.type = 28
      )
      AND d.trans_date >= ?
      AND d.trans_date < ?
    GROUP BY a.type
ENDOF
  my $total = 0;
  $self->debug($year.sprintf('%02d',$month).$one.":". $year2.sprintf('%02d',$month2).$one);
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[0], plug => [ $l, $year.sprintf('%02d',$month).$one, $year2.sprintf('%02d',$month2).$one ]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    $self->{'TYPE_'.$data->{type}} = $data->{amount};
    if ($data->{amount} ne '') { $total += $data->{amount}; }
  }
  $self->set('total', $total);
  return 1;
}

sub javascript
{
  my $self = shift;
  $self->select;
  my %form;
  $form{javascript} .= " PAYROLL['total'] = '". $self->get('total')."';\n";
  $form{javascript} .= " PAYROLL['quarter'] = '". $self->get('quarter')."';\n";
  $form{javascript} .= " PAYROLL['24t'] = '". $self->{TYPE_24}."';\n";
  $form{javascript} .= " PAYROLL['26t'] = '". $self->{TYPE_26}."';\n";
  $form{javascript} .= " PAYROLL['27t'] = '". $self->{TYPE_27}."';\n";
  $form{javascript} .= " PAYROLL['28t'] = '". $self->{TYPE_28}."';\n";
  $form{javascript} .= " PAYROLL['24'] = '". $self->get('24')."';\n";
  $form{javascript} .= " PAYROLL['26'] = '". $self->get('26')."';\n";
  $form{javascript} .= " PAYROLL['27'] = '". $self->get('27')."';\n";
  $form{javascript} .= " PAYROLL['28'] = '". $self->get('28')."';\n";
  return %form;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
