# MO941.pm - The Object Class that provides a MO941 Object
#$Id: MO941.pm,v 1.8.6.1 2006/07/14 20:05:59 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Payroll::MO941;
use strict;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Objects::Account;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.8.6.1 $';

=head1 NAME

MO941 - Object used to build a MO941 Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::MO941;
  my $MO941Obj = Portal::Accounting::Objects::MO941->new(   );
  if ($MO941Obj->error())
  {
    die $MO941Obj->errorMessage();
  }

=head1 DESCRIPTION

MO941 holds forms for dealing with Federal 941 payroll reports.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::MO941.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  $self->{ACCOUNT} = Portal::Accounting::Objects::Account->new(langObj => $self->langObj, dbHandle=> $self->dbHandle, ledgerCode => $args{ledgerCode});
  my $accounts = {names => [], values => []};
  if ($self->{ACCOUNT}->error) { $self->error($self->{ACCOUNT}->errorMessage); }
  else
  {
    $self->{ACCOUNT}->set('type', '25');
    $accounts = $self->{ACCOUNT}->selectOptionsByType;
  }
  my @date = Today();
  my $quarter = 4;
  if ($date[1] > 3) { $quarter = 1; }
  if ($date[1] > 6) { $quarter = 2; }
  if ($date[1] > 9) { $quarter = 3; }
  my $l = $self->{langObj};
  my $Auto = $l->map('Auto');
  my $Manual = $l->map('Manual');
  $self->{data}->{form} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll::Report" },
    "command" => { -Type => "hidden", -Value => "report" },
    "report" => { -Type => "hidden", -Value => "MO941" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "name" => { -Type => "text", -DataType => "string", -Label => $l->map('Subtitle'), },
    account => { -Type => 'select', -DataType => 'int', -Label => 'State Tax Payable Account', -Options => $accounts, },
    "year" => {-Type => "text", size => 4, -Label => $l->map('Year'),
      -DataType => "integer", -Value => $date[0],
    },
    "quarter" => {-Type => "select", -Label => $l->map('Quarter'),
      -DataType => "string", -Value => $quarter,
      -Options => { names => [qw(1 2 3 4)], values => [qw(1 2 3 4)] },
    },
    "from" => { -Label => $l->map("From"), -Type => "date-picker", -DataType => "string",
      -year => $date[0] ,
    },
    "to" => { -Label => $l->map("To"),   -Type => "date-picker", -DataType => "string",
      -year => $date[0] ,
    },
    "previous" => {-Type => "calculator", size => 10, -Label => $l->map('MOpreviousOver'),
      -DataType => "float", -undoButton => 0,
    },
    "additions" => {-Type => "calculator", size => 10, -Label => $l->map('Additions to Tax'),
      -DataType => "float", -undoButton => 0,
    },
    "interest" => {-Type => "calculator", size => 10, -Label => $l->map('Interest'),
      -DataType => "float", -undoButton => 0,
    },
    "compensation" => {-Type => "calculator", size => 10, -Label => $l->map('Compensation Deduction'),
      -DataType => "float", -undoButton => 0,
    },
    "auto" => {-Type => "radio", -DataType => "integer", -Value => 'auto', -Label => $l->map('Compensation Deduction'),
      -Options => { names => [ $Auto, $Manual ], values => [ qw(auto manual) ] }
    },
  };
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll" },
    "command" => { -Type => "hidden", -Value => "detail" },
    "report" => { -Type => "hidden", -Value => "MO941" },
    "auto" => { -Type => "hidden", -Value => "auto" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "name" => { -Type => "text", -DataType => "string", -ReadOnly => 1, },
    "year" => {-Type => "text", size => 4, -Label => $l->map('Year'),
      -DataType => "integer", -Value => $date[0],
    },
    "quarter" => {-Type => "select", -Label => $l->map('Quarter'),
      -DataType => "string", -Value => $quarter,
      -Options => { names => [qw(1 2 3 4)], values => [qw(1 2 3 4)] },
    },
    "from" => {-Type => "text", -Label => $l->map('From'),
      -DataType => "text", -ReadOnly => 1,
    },
    "to" => {-Type => "text", -Label => $l->map('To'),
      -DataType => "text", -ReadOnly => 1,
    },
    "previous" => {-Type => "text", -Label => $l->map('MOpreviousOver'),
      -DataType => "float", -ReadOnly => 1,
    },
    "additions" => {-Type => "text", -Label => $l->map('Additions to Tax'),
      -DataType => "float", -ReadOnly => 1,
    },
    "interest" => {-Type => "text", -Label => $l->map('Interest'),
      -DataType => "float",  -ReadOnly => 1,
    },
    "compensation" => {-Type => "text", -Label => $l->map('Compensation Deduction'),
      -DataType => "float",  -ReadOnly => 1,
    },
    "withholding" => {-Type => "text", -Label => $l->map('Withholding This Period'),
      -DataType => "float",  -ReadOnly => 1,
    },
    "balance" => {-Type => "text", -Label => $l->map('Balance Due'),
      -DataType => "float",  -ReadOnly => 1,
    },
    "total" => {-Type => "text", -Label => $l->map('Total Amount Due'),
      -DataType => "float",  -ReadOnly => 1,
    },
    "stateid" => { -Type => "text", -ReadOnly => 1, -Value => '' },
    "fedid" => { -Type => "text", -ReadOnly => 1, -Value => '' },
  };
  $self->{css}->{form} .= "div { margin: .3em; }";
   $self->{template}->{form} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=report#
  <div class="primary">
    <div>
      #LABEL=name# #FI=name#
    </div>
    <div>
      #LABEL=year#
      #FIR=year#
      #LABEL=quarter#
      #FIR=quarter#
    </div>
    <div>
      #LABEL=account#
      #FIR=account#
    </div>
    <div style="border-width: 1px; border-style: solid; float: left; padding: .5em; margin: .5em; ">
      <i>#LANG=Whole numbers only#</i>
      <div style="">
        #LABEL=compensation#<br>
        <div style="float: left;">#FIR=auto#</div>
        <br>#FIR=compensation#
      </div>
      <div style="clear: both;">#FIR=previous#  #LABEL=previous#</div>
      <div>#FIR=additions# #LABEL=additions#</div>
      <div>#FIR=interest#  #LABEL=interest#</div>
    </div>
    <div style="clear: both;"></div>
    <input type="submit" name="submit" value="#LANG=Submit#" />
  </div>
</form>
  #FORMREQUIREDSTRING#
ENDOF
   $self->{template}->{main} = <<"ENDOF";
<b>#LANG=Federal Tax ID#: #FIELD=fedid#<br>#LANG=State Tax ID#: #FIELD=stateid#</b>
<table style="width: 25em;">
  <tr>
    <td colspan="2">
      <b>#FI=name#</b>
    </td>
  </tr>
  <tr>
    <td colspan="2">
      <b>#LABEL=from#
      #FI=from#
      #LABEL=to#
      #FI=to#</b>
    </td>
  </tr>
  <tr>
    <td align="left">#LABEL=withholding#</td>
    <td align="right">#FIELD=withholding#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=compensation#</td>
    <td align="right">-#FIELD=compensation#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=previous#</td>
    <td align="right">#FIELD=previous#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=balance#</td>
    <td align="right">#FIELD=balance#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=additions#</td>
    <td align="right">#FIELD=additions#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=interest#</td>
    <td align="right">#FIELD=interest#</td>
  </tr>
  <tr>
    <td align="left">#LABEL=total#</td>
    <td align="right">#FIELD=total#</td>
  </tr>
</table>
ENDOF
  $self->{javascript}->{form} = <<"ENDOF";
ENDOF
  $self->{profile}->{form} = {
    required => [ qw( quarter year account ) ],
    optional => [ qw( auto name previous additions interest compensation ) ],
    constraints => {
      from => qr/^\d{4}-\d{2}-\d{2}$/,
      to => qr/^\d{4}-\d{2}-\d{2}$/,
      previous => qr/^-?\d+$/,
      additions => qr/^\d+$/,
      interest => qr/^\d+$/,
      compensation => qr/^\d+$/,
      account => qr/^\d+$/,
    },
  };
  $self->{profile}->{main} = {
    required => [ qw( quarter year ) ],
    optional => [ qw( auto name previous additions interest compensation ) ],
    constraints => {
      from => qr/^\d{4}-\d{2}-\d{2}$/,
      to => qr/^\d{4}-\d{2}-\d{2}$/,
      previous => qr/^\d+\.?\d*$/,
      additions => qr/^\d+\.?\d*$/,
      interest => qr/^\d+\.?\d*$/,
      compensation => qr/^\d+\.?\d*$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  Gathers data from database for this report.

=cut
sub select
{
  my $self = shift;
  my $l = $self->get('ledgerCode');
  my $year = $self->get('year');
  my $q = $self->get('quarter');
  my $to = '-03-31';
  my $from = '-01-01';
  my $total = 0;
  if ($q == 4)
  {
    $from = '-10-01';
    $to = '-12-31';
  }
  elsif ($q == 3)
  {
    $from = '-07-01';
    $to = '-09-30';
  }
  elsif ($q == 2)
  {
    $from = '-04-01';
    $to = '-06-30';
  }

  my $to = $year.$to;
  my $from = $year.$from;
  $self->set('from',$from);
  $self->set('to',$to);
  my $l = $self->get('ledgerCode');

  $self->{LIST}[0] = <<"ENDOF";
  SELECT a.name, a.code, d.account_num AS account, to_char(SUM(d.d_amount), '9999990.00') AS amount
  FROM sub_acc_tb AS s, debitcredit_tb AS d, account_tb AS a
  WHERE d.ledger_code = ?
    AND s.type = 120
    AND s.sub_acc_number = d.sub_account_num
    AND d.account_num = a.number
    AND a.type = 25
    AND d.trans_date BETWEEN ? AND ?
  GROUP BY d.account_num, a.name, a.code
  ORDER BY a.name, d.account_num
ENDOF
  my $sth = $self->{dbHandle}->read(sql => $self->{LIST}[0], plug => [$self->get('ledgerCode'), $from, $to ]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    push (@{$self->{DATA}->{$data->{name}}}, $data);
    $self->set('withholding', sprintf("%.02f", $self->get('withholding') + $data->{amount}) );
  }
  $self->{LIST}[1] = <<"ENDOF";
  SELECT fedid, stateid
  FROM ledger_tb
  WHERE code = '$l'
ENDOF
  my $sth = $self->{dbHandle}->read($self->{LIST}[1]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $sth->fetchrow_hashref)
  {
    $self->set('fedid', $data->{fedid});
    $self->set('stateid', $data->{stateid});
  }
  return 1;
}

sub printReport
{
  my $self = shift;
  #calculate compensation
  if ($self->get('auto') eq 'auto')
  {
    my $foo = $self->{data}->{main}->{withholding}->{-Value};
    my $bar;
    if ($foo > 10000)
    {
      $bar = sprintf("%0d",($foo - 10000) * 0.005 + 5000 * 0.01 + 5000 * 0.02 + .5);
    }
    elsif ($foo > 5000)
    {
      $bar = sprintf("%0d", ($foo - 5000) * 0.01 + 5000 * 0.02 + .05);
    }
    else
    {
      $bar = sprintf("%0d", $foo * 0.02 + .5);
    }
    $self->set('compensation', sprintf("%.02f",$bar) );
  }
  #calculate balance and total
  my $balance = $self->get('withholding') - $self->get('compensation') + $self->get('previous');
  $self->set('balance', sprintf("%.02f", $balance) );
  $self->set('total', sprintf("%.02f", $balance + $self->get('additions') + $self->get('interest')) );
  return $self->printForm(form => 'main');
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
